@@ -39,9 +39,11 @@
 
 #include "qcomponentaddedchange.h"
 #include "qcomponentaddedchange_p.h"
+
 #include <Qt3DCore/qcomponent.h>
 #include <Qt3DCore/qentity.h>
-#include <private/qnode_p.h>
+
+#include <Qt3DCore/private/qnode_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -58,6 +60,7 @@ QComponentAddedChangePrivate::QComponentAddedChangePrivate(const QEntity *entity
 
 /*!
  * \class Qt3DCore::QComponentAddedChange
+ * \inheaderfile Qt3DCore/QComponentAddedChange
  * \inherits Qt3DCore::QSceneChange
  * \inmodule Qt3DCore
  * \brief The QComponentAddedChange class is used to notify when a component is added to an entity
@@ -72,7 +75,7 @@ QComponentAddedChangePrivate::QComponentAddedChangePrivate(const QEntity *entity
  */
 
 /*!
- * Constructs a new QComponentAddedChange with with \a entity and  \a component.
+ * Constructs a new QComponentAddedChange which will notify \a entity that \a component was added
  */
 QComponentAddedChange::QComponentAddedChange(const QEntity *entity,
                                              const QComponent *component)
@@ -81,6 +84,16 @@ QComponentAddedChange::QComponentAddedChange(const QEntity *entity,
 {
 }
 
+/*!
+ * Constructs a new QComponentAddedChange which will notify \a component that it was added to \a entity
+ */
+QComponentAddedChange::QComponentAddedChange(const QComponent *component,
+                                             const QEntity *entity)
+    : QSceneChange(*new QComponentAddedChangePrivate(entity, component),
+                   ComponentAdded, component->id())
+{
+}
+
 QComponentAddedChange::~QComponentAddedChange()
 {
 }
