@@ -39,6 +39,7 @@
 
 #include "qpropertynodeaddedchange.h"
 #include "qpropertynodeaddedchange_p.h"
+
 #include <Qt3DCore/private/qnode_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -53,6 +54,7 @@ QPropertyNodeAddedChangePrivate::QPropertyNodeAddedChangePrivate()
 
 /*!
  * \class Qt3DCore::QPropertyNodeAddedChange
+ * \inheaderfile Qt3DCore/QPropertyNodeAddedChange
  * \inherits Qt3DCore::QStaticPropertyValueAddedChangeBase
  * \inmodule Qt3DCore
  * \brief Used to notify when a node is added to a property
@@ -73,7 +75,17 @@ QPropertyNodeAddedChange::QPropertyNodeAddedChange(QNodeId subjectId, QNode *nod
     : QStaticPropertyValueAddedChangeBase(*new QPropertyNodeAddedChangePrivate, subjectId)
 {
     Q_D(QPropertyNodeAddedChange);
+    Q_ASSERT(node);
     d->m_addedNodeIdTypePair = QNodeIdTypePair(node->id(), QNodePrivate::findStaticMetaObject(node->metaObject()));
+
+    // Ensure the node has issued a node creation change. We can end
+    // up here if a newly created node with a parent is immediately set
+    // as a property on another node. In this case the deferred call to
+    // _q_postConstructorInit() will not have happened yet as the event
+    // loop will still be blocked. So force it here and we catch this
+    // eventuality in the _q_postConstructorInit() function so that we
+    // do not repeat the creation and new child scene change events.
+    QNodePrivate::get(node)->_q_postConstructorInit();
 }
 
 /*! \internal */
