@@ -39,9 +39,11 @@
 
 #include "qbackendnode.h"
 #include "qbackendnode_p.h"
-#include "qaspectengine.h"
-#include "qnode.h"
-#include "qpropertyupdatedchange.h"
+
+#include <Qt3DCore/qaspectengine.h>
+#include <Qt3DCore/qnode.h>
+#include <Qt3DCore/qpropertyupdatedchange.h>
+
 #include <Qt3DCore/private/corelogging_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -94,6 +96,7 @@ QBackendNodePrivate *QBackendNodePrivate::get(QBackendNode *n)
 
 /*!
  * \class Qt3DCore::QBackendNodeMapper
+ * \inheaderfile Qt3DCore/QBackendNodeMapper
  * \inmodule Qt3DCore
  *
  * \brief Creates and maps backend nodes to their respective frontend nodes
@@ -123,6 +126,7 @@ QBackendNodePrivate *QBackendNodePrivate::get(QBackendNode *n)
 
 /*!
  * \class Qt3DCore::QBackendNode
+ * \inheaderfile Qt3DCore/QBackendNode
  * \inmodule Qt3DCore
  *
  * \brief The base class for all Qt3D backend nodes
@@ -229,7 +233,7 @@ void QBackendNode::sceneChangeEvent(const QSceneChangePtr &e)
     switch (e->type()) {
         case PropertyUpdated: {
             if (propertyChange->propertyName() == QByteArrayLiteral("enabled"))
-                d->m_enabled = propertyChange->value().value<bool>();
+                d->m_enabled = propertyChange->value().toBool();
             break;
         }
         default:
