@@ -39,10 +39,14 @@
 
 #include "qcomponent.h"
 #include "qcomponent_p.h"
-#include "qentity.h"
-#include "qentity_p.h"
-#include "qscene_p.h"
+
 #include <Qt3DCore/qpropertyupdatedchange.h>
+#include <Qt3DCore/qcomponentaddedchange.h>
+#include <Qt3DCore/qcomponentremovedchange.h>
+#include <Qt3DCore/qentity.h>
+
+#include <Qt3DCore/private/qentity_p.h>
+#include <Qt3DCore/private/qscene_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -60,6 +64,7 @@ QComponentPrivate::~QComponentPrivate()
 
 void QComponentPrivate::addEntity(QEntity *entity)
 {
+    Q_Q(QComponent);
     m_entities.append(entity);
 
     if (m_scene != nullptr && !m_scene->hasEntityForComponent(m_id, entity->id())) {
@@ -68,17 +73,22 @@ void QComponentPrivate::addEntity(QEntity *entity)
         m_scene->addEntityForComponent(m_id, entity->id());
     }
 
-    // TODO: Add QAddedToEntityChange to be delivered to components on the backend
+    const auto componentAddedChange = QComponentAddedChangePtr::create(q, entity);
+    notifyObservers(componentAddedChange);
+    Q_EMIT q->addedToEntity(entity);
 }
 
 void QComponentPrivate::removeEntity(QEntity *entity)
 {
-    // TODO: Add QRemovedFromEntityChange to be delivered to components on the backend
-
+    Q_Q(QComponent);
     if (m_scene != nullptr)
         m_scene->removeEntityForComponent(m_id, entity->id());
 
     m_entities.removeAll(entity);
+
+    const auto componentRemovedChange = QComponentRemovedChangePtr::create(q, entity);
+    notifyObservers(componentRemovedChange);
+    Q_EMIT q->removedFromEntity(entity);
 }
 
 /*!
