@@ -40,25 +40,26 @@
 #include "qnode.h"
 #include "qnode_p.h"
 
-#include <Qt3DCore/qentity.h>
+#include <Qt3DCore/QComponent>
+#include <Qt3DCore/qaspectengine.h>
 #include <Qt3DCore/qdynamicpropertyupdatedchange.h>
-#include <Qt3DCore/qpropertyupdatedchange.h>
+#include <Qt3DCore/qentity.h>
+#include <Qt3DCore/qnodedestroyedchange.h>
 #include <Qt3DCore/qpropertynodeaddedchange.h>
 #include <Qt3DCore/qpropertynoderemovedchange.h>
-#include <Qt3DCore/qnodedestroyedchange.h>
-#include <Qt3DCore/qaspectengine.h>
-#include <Qt3DCore/private/qdestructionidandtypecollector_p.h>
-#include <Qt3DCore/private/qscene_p.h>
-#include <Qt3DCore/private/qpostman_p.h>
-#include <QEvent>
-#include <QChildEvent>
-#include <QMetaObject>
-#include <QMetaProperty>
-#include <QtCore/private/qmetaobject_p.h>
-#include <Qt3DCore/QComponent>
+#include <Qt3DCore/qpropertyupdatedchange.h>
+#include <QtCore/QChildEvent>
+#include <QtCore/QEvent>
+#include <QtCore/QMetaObject>
+#include <QtCore/QMetaProperty>
+
 #include <Qt3DCore/private/corelogging_p.h>
+#include <Qt3DCore/private/qdestructionidandtypecollector_p.h>
 #include <Qt3DCore/private/qnodecreatedchangegenerator_p.h>
 #include <Qt3DCore/private/qnodevisitor_p.h>
+#include <Qt3DCore/private/qpostman_p.h>
+#include <Qt3DCore/private/qscene_p.h>
+#include <QtCore/private/qmetaobject_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -73,6 +74,8 @@ QNodePrivate::QNodePrivate()
     , m_blockNotifications(false)
     , m_hasBackendNode(false)
     , m_enabled(true)
+    , m_notifiedParent(false)
+    , m_defaultPropertyTrackMode(QNode::TrackFinalValues)
     , m_propertyChangesSetup(false)
     , m_signals(this)
 {
@@ -97,10 +100,8 @@ void QNodePrivate::init(QNode *parent)
     m_scene = parentPrivate->m_scene;
     Q_Q(QNode);
     if (m_scene) {
-        m_scene->addObservable(q); // Sets the m_changeArbiter to that of the scene
-
-        // Scehdule the backend notification
-        QMetaObject::invokeMethod(q, "_q_notifyCreationAndChildChanges", Qt::QueuedConnection);
+        // schedule the backend notification and scene registering -> set observers through scene
+        QMetaObject::invokeMethod(q, "_q_postConstructorInit", Qt::QueuedConnection);
     }
 }
 
@@ -114,7 +115,7 @@ void QNodePrivate::notifyCreationChange()
     Q_Q(QNode);
     // Do nothing if we already have already sent a node creation change
     // and not a subsequent node destroyed change.
-    if (m_hasBackendNode)
+    if (m_hasBackendNode || !m_scene)
         return;
     QNodeCreatedChangeGenerator generator(q);
     const auto creationChanges = generator.creationChanges();
@@ -166,16 +167,31 @@ void QNodePrivate::notifyDestructionChangesAndRemoveFromScene()
  * parent backend node of its new child. This is called in a deferred manner
  * by the QNodePrivate::init() method to notify the backend of newly created
  * nodes with a parent that is already part of the scene.
+ *
+ * Also notify the scene of this node, so it may set it's change arbiter.
  */
-void QNodePrivate::_q_notifyCreationAndChildChanges()
+void QNodePrivate::_q_postConstructorInit()
 {
     Q_Q(QNode);
 
+    // If we've already done the work then bail out. This can happen if the
+    // user creates a QNode subclass with an explicit parent, then immediately
+    // sets the new QNode as a property on another node. In this case, the
+    // property setter will call this function directly, but as we can't
+    // un-schedule a deferred invocation, this function will be called again
+    // the next time the event loop spins. So, catch this case and abort.
+    if (m_hasBackendNode)
+        return;
+
     // Check that the parent hasn't been unset since this call was enqueued
     auto parentNode = q->parentNode();
     if (!parentNode)
         return;
 
+
+    if (m_scene)
+        m_scene->addObservable(q); // Sets the m_changeArbiter to that of the scene
+
     // Let the backend know we have been added to the scene
     notifyCreationChange();
 
@@ -187,7 +203,7 @@ void QNodePrivate::_q_notifyCreationAndChildChanges()
 /*!
  * \internal
  *
- * Called by _q_setParentHelper() or _q_notifyCreationAndChildChanges()
+ * Called by _q_setParentHelper() or _q_postConstructorInit()
  * on the main thread.
  */
 void QNodePrivate::_q_addChild(QNode *childNode)
@@ -195,13 +211,19 @@ void QNodePrivate::_q_addChild(QNode *childNode)
     Q_ASSERT(childNode);
     Q_ASSERT_X(childNode->parent() == q_func(), Q_FUNC_INFO,  "not a child of this node");
 
+    // Have we already notified the parent about its new child? If so, bail out
+    // early so that we do not send more than one new child event to the backend
+    QNodePrivate *childD = QNodePrivate::get(childNode);
+    if (childD->m_notifiedParent == true)
+        return;
+
     // Store our id as the parentId in the child so that even if the child gets
     // removed from the scene as part of the destruction of the parent, when the
     // parent's children are deleted in the QObject dtor, we still have access to
     // the parentId. If we didn't store this, we wouldn't have access at that time
-    // because the parent woudl then only be a QObject, the QNode part would have
+    // because the parent would then only be a QObject, the QNode part would have
     // been destroyed already.
-    QNodePrivate::get(childNode)->m_parentId = m_id;
+    childD->m_parentId = m_id;
 
     if (!m_scene)
         return;
@@ -209,6 +231,11 @@ void QNodePrivate::_q_addChild(QNode *childNode)
     // We need to send a QPropertyNodeAddedChange to the backend
     // to notify the backend that we have a new child
     if (m_changeArbiter != nullptr) {
+        // Flag that we have notified the parent. We do this immediately before
+        // creating the change because that recurses back into this function and
+        // we need to catch that to avoid sending more than one new child event
+        // to the backend.
+        childD->m_notifiedParent = true;
         const auto change = QPropertyNodeAddedChangePtr::create(m_id, childNode);
         change->setPropertyName("children");
         notifyObservers(change);
@@ -284,6 +311,9 @@ void QNodePrivate::_q_setParentHelper(QNode *parent)
             notifyDestructionChangesAndRemoveFromScene();
     }
 
+    // Flag that we need to notify any new parent
+    m_notifiedParent = false;
+
     // Basically QObject::setParent but for QObjectPrivate
     QObjectPrivate::setParent_helper(parent);
     QNode *newParentNode = q->parentNode();
@@ -300,7 +330,21 @@ void QNodePrivate::_q_setParentHelper(QNode *parent)
                 visitor.traverse(q, newParentNode->d_func(), &QNodePrivate::setSceneHelper);
             }
 
-            notifyCreationChange();
+            // We want to make sure that subTreeRoot is always created before
+            // child.
+            // Given a case such as below
+            // QEntity *subTreeRoot = new QEntity(someGlobalExisitingRoot)
+            // QEntity *child = new QEntity();
+            // child->setParent(subTreeRoot)
+            // We need to take into account that subTreeRoot needs to be
+            // created in the backend before the child.
+            // Therefore we only call notifyCreationChanges if the parent
+            // hasn't been created yet as we know that when the parent will be
+            // fully created, it will also send the changes for all of its
+            // children
+
+            if (QNodePrivate::get(newParentNode)->m_hasBackendNode)
+                notifyCreationChange();
         }
 
         // If we have a valid new parent, we let him know that we are its child
@@ -344,17 +388,43 @@ void QNodePrivate::propertyChanged(int propertyIndex)
     if (m_blockNotifications)
         return;
 
+    const auto toBackendValue = [this](const QVariant &data) -> QVariant
+    {
+        if (data.canConvert<QNode*>()) {
+            QNode *node = data.value<QNode*>();
+
+            // Ensure the node has issued a node creation change. We can end
+            // up here if a newly created node with a parent is immediately set
+            // as a property on another node. In this case the deferred call to
+            // _q_postConstructorInit() will not have happened yet as the event
+            // loop will still be blocked. So force it here and we catch this
+            // eventuality in the _q_postConstructorInit() function so that we
+            // do not repeat the creation and new child scene change events.
+            if (node)
+                QNodePrivate::get(node)->_q_postConstructorInit();
+
+            const QNodeId id = node ? node->id() : QNodeId();
+            return QVariant::fromValue(id);
+        }
+
+        return data;
+    };
+
     Q_Q(QNode);
 
     const QMetaProperty property = q->metaObject()->property(propertyIndex);
 
     const QVariant data = property.read(q);
-    if (data.canConvert<QNode*>()) {
-        const QNode * const node = data.value<QNode*>();
-        const QNodeId id = node ? node->id() : QNodeId();
-        notifyPropertyChange(property.name(), QVariant::fromValue(id));
+
+    if (data.type() == QVariant::List) {
+        QSequentialIterable iterable = data.value<QSequentialIterable>();
+        QVariantList variants;
+        variants.reserve(iterable.size());
+        for (const auto &v : iterable)
+            variants.append(toBackendValue(v));
+        notifyPropertyChange(property.name(), variants);
     } else {
-        notifyPropertyChange(property.name(), data);
+        notifyPropertyChange(property.name(), toBackendValue(data));
     }
 }
 
@@ -389,20 +459,22 @@ void QNodePrivate::setSceneHelper(QNode *root)
     Recursively unsets and remove nodes in the subtree of base node \a root from
     the scene. Also takes care of removing Components and Entities connections.
  */
-void QNodePrivate::unsetSceneHelper(QNode *root)
+void QNodePrivate::unsetSceneHelper(QNode *node)
 {
+    QNodePrivate *nodePrivate = QNodePrivate::get(node);
+
     // We also need to handle QEntity <-> QComponent relationships removal
-    if (QComponent *c = qobject_cast<QComponent *>(root)) {
+    if (QComponent *c = qobject_cast<QComponent *>(node)) {
         const QVector<QEntity *> entities = c->entities();
         for (QEntity *entity : entities) {
-            if (m_scene)
-                m_scene->removeEntityForComponent(c->id(), entity->id());
+            if (nodePrivate->m_scene)
+                nodePrivate->m_scene->removeEntityForComponent(c->id(), entity->id());
         }
     }
 
-    if (m_scene != nullptr)
-        m_scene->removeObservable(root);
-    root->d_func()->setScene(nullptr);
+    if (nodePrivate->m_scene != nullptr)
+        nodePrivate->m_scene->removeObservable(node);
+    nodePrivate->setScene(nullptr);
 }
 
 /*!
@@ -484,7 +556,23 @@ void QNode::notifyObservers(const QSceneChangePtr &change)
 void QNode::sceneChangeEvent(const QSceneChangePtr &change)
 {
     Q_UNUSED(change);
-    qWarning() << Q_FUNC_INFO << "sceneChangeEvent should have been subclassed";
+    if (change->type() == Qt3DCore::PropertyUpdated) {
+        // TODO: Do this more efficiently. We could pass the metaobject and property
+        //       index to the animation aspect via the QChannelMapping. This would
+        //       allow us to avoid the propertyIndex lookup here by sending them in
+        //       a new subclass of QPropertyUpdateChange.
+        // Try to find property and call setter
+        auto e = qSharedPointerCast<Qt3DCore::QPropertyUpdatedChange>(change);
+        const QMetaObject *mo = metaObject();
+        const int propertyIndex = mo->indexOfProperty(e->propertyName());
+        QMetaProperty mp = mo->property(propertyIndex);
+        bool wasBlocked = blockNotifications(true);
+        mp.write(this, e->value());
+        blockNotifications(wasBlocked);
+    } else {
+        // Nothing is handling this change, warn the user.
+        qWarning() << Q_FUNC_INFO << "sceneChangeEvent should have been subclassed";
+    }
 }
 
 /*!
@@ -492,8 +580,13 @@ void QNode::sceneChangeEvent(const QSceneChangePtr &change)
  */
 void QNodePrivate::setScene(QScene *scene)
 {
-    if (m_scene != scene)
+    if (m_scene != scene) {
+        if (m_scene != nullptr)
+            m_scene->removePropertyTrackDataForNode(m_id);
         m_scene = scene;
+        // set PropertyTrackData in the scene
+        updatePropertyTrackMode();
+    }
 }
 
 /*!
@@ -577,6 +670,16 @@ void QNodePrivate::insertTree(QNode *treeRoot, int depth)
         treeRoot->setParent(q_func());
 }
 
+void QNodePrivate::updatePropertyTrackMode()
+{
+    if (m_scene != nullptr) {
+        QScene::NodePropertyTrackData trackData;
+        trackData.defaultTrackMode = m_defaultPropertyTrackMode;
+        trackData.trackedPropertiesOverrides = m_trackedPropertiesOverrides;
+        m_scene->setPropertyTrackDataForNode(m_id, trackData);
+    }
+}
+
 /*!
     \internal
  */
@@ -766,6 +869,20 @@ void QNode::setEnabled(bool isEnabled)
     emit enabledChanged(isEnabled);
 }
 
+void QNode::setDefaultPropertyTrackingMode(QNode::PropertyTrackingMode mode)
+{
+    Q_D(QNode);
+    if (d->m_defaultPropertyTrackMode == mode)
+        return;
+
+    d->m_defaultPropertyTrackMode = mode;
+    // The backend doesn't care about such notification
+    const bool blocked = blockNotifications(true);
+    emit defaultPropertyTrackingModeChanged(mode);
+    blockNotifications(blocked);
+    d->updatePropertyTrackMode();
+}
+
 /*!
     \property Qt3DCore::QNode::enabled
 
@@ -782,6 +899,48 @@ bool QNode::isEnabled() const
     return d->m_enabled;
 }
 
+/*!
+    \property Qt3DCore::QNode::defaultPropertyTrackingMode
+
+    Holds the default property tracking mode which determines whether a QNode should
+    be listening for property updates. This only applies to properties which
+    haven't been overridden by a call to setPropertyTracking.
+
+    By default it is set to QNode::TrackFinalValues
+*/
+QNode::PropertyTrackingMode QNode::defaultPropertyTrackingMode() const
+{
+    Q_D(const QNode);
+    return d->m_defaultPropertyTrackMode;
+}
+
+void QNode::setPropertyTracking(const QString &propertyName, QNode::PropertyTrackingMode trackMode)
+{
+    Q_D(QNode);
+    d->m_trackedPropertiesOverrides.insert(propertyName, trackMode);
+    d->updatePropertyTrackMode();
+}
+
+QNode::PropertyTrackingMode QNode::propertyTracking(const QString &propertyName) const
+{
+    Q_D(const QNode);
+    return d->m_trackedPropertiesOverrides.value(propertyName, d->m_defaultPropertyTrackMode);
+}
+
+void QNode::clearPropertyTracking(const QString &propertyName)
+{
+    Q_D(QNode);
+    d->m_trackedPropertiesOverrides.remove(propertyName);
+    d->updatePropertyTrackMode();
+}
+
+void QNode::clearPropertyTrackings()
+{
+    Q_D(QNode);
+    d->m_trackedPropertiesOverrides.clear();
+    d->updatePropertyTrackMode();
+}
+
 QNodeCreatedChangeBasePtr QNode::createNodeCreationChange() const
 {
     // Uncomment this when implementing new frontend and backend types.
