@@ -39,6 +39,7 @@
 
 /*!
  * \class Qt3DExtras::QConeGeometry
+ * \inheaderfile Qt3DExtras/QConeGeometry
  * \inmodule Qt3DExtras
  * \brief The QConeGeometry class allows creation of a cone in 3D space.
  * \since 5.7
@@ -56,10 +57,12 @@
 
 #include "qconegeometry.h"
 #include "qconegeometry_p.h"
+
 #include <Qt3DRender/qbuffer.h>
 #include <Qt3DRender/qbufferdatagenerator.h>
 #include <Qt3DRender/qattribute.h>
-#include <QVector3D>
+#include <QtGui/QVector3D>
+
 #include <cmath>
 
 QT_BEGIN_NAMESPACE
