@@ -42,13 +42,13 @@
 
 #include <QtQml/QQmlContext>
 #include <QtQml/QQmlComponent>
-#include <QtQml/QQmlInfo>
 #include <QtQml/QQmlError>
-#include <QtQml/private/qqmlobjectmodel_p.h>
+#include <QtQml/QQmlInfo>
+
+#include <Qt3DCore/private/qnode_p.h>
+#include <QtQml/private/qqmlchangeset_p.h>
 #include <QtQml/private/qqmldelegatemodel_p.h>
-#include <private/qnode_p.h>
-#include <private/qqmlchangeset_p.h>
-#include <private/qqmlobjectmodel_p.h>
+#include <QtQml/private/qqmlobjectmodel_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -98,7 +98,8 @@ Quick3DNodeInstantiatorPrivate::Quick3DNodeInstantiatorPrivate()
 
 Quick3DNodeInstantiatorPrivate::~Quick3DNodeInstantiatorPrivate()
 {
-    qDeleteAll(m_objects);
+    if (m_ownModel)
+        delete m_instanceModel;
 }
 
 void Quick3DNodeInstantiatorPrivate::clear()
@@ -110,11 +111,11 @@ void Quick3DNodeInstantiatorPrivate::clear()
         return;
 
     for (int i = 0; i < m_objects.count(); i++) {
-        q->objectRemoved(i, m_objects[i]);
+        emit q->objectRemoved(i, m_objects[i]);
         m_instanceModel->release(m_objects[i]);
     }
     m_objects.clear();
-    q->objectChanged();
+    emit q->objectChanged();
 }
 
 void Quick3DNodeInstantiatorPrivate::regenerate()
@@ -129,18 +130,19 @@ void Quick3DNodeInstantiatorPrivate::regenerate()
 
     if (!m_active || !m_instanceModel || !m_instanceModel->count() || !m_instanceModel->isValid()) {
         if (prevCount)
-            q->countChanged();
+            emit q->countChanged();
         return;
     }
 
     for (int i = 0; i < m_instanceModel->count(); i++) {
-        QObject *object = m_instanceModel->object(i, m_async);
+        QObject *object = m_instanceModel->object(i, m_async ?
+            QQmlIncubator::Asynchronous : QQmlIncubator::AsynchronousIfNested);
         // If the item was already created we won't get a createdItem
         if (object)
             _q_createdItem(i, object);
     }
     if (q->count() != prevCount)
-        q->countChanged();
+        emit q->countChanged();
 }
 
 void Quick3DNodeInstantiatorPrivate::_q_createdItem(int idx, QObject *item)
@@ -151,8 +153,8 @@ void Quick3DNodeInstantiatorPrivate::_q_createdItem(int idx, QObject *item)
     static_cast<QNode *>(item)->setParent(q->parentNode());
     m_objects.insert(idx, item);
     if (m_objects.count() == 1)
-        q->objectChanged();
-    q->objectAdded(idx, item);
+        emit q->objectChanged();
+    emit q->objectAdded(idx, item);
 }
 
 void Quick3DNodeInstantiatorPrivate::_q_modelUpdated(const QQmlChangeSet &changeSet, bool reset)
@@ -165,7 +167,7 @@ void Quick3DNodeInstantiatorPrivate::_q_modelUpdated(const QQmlChangeSet &change
     if (reset) {
         regenerate();
         if (changeSet.difference() != 0)
-            q->countChanged();
+            emit q->countChanged();
         return;
     }
 
@@ -184,7 +186,7 @@ void Quick3DNodeInstantiatorPrivate::_q_modelUpdated(const QQmlChangeSet &change
             while (count--) {
                 QObject *obj = m_objects.at(index);
                 m_objects.remove(index);
-                q->objectRemoved(index, obj);
+                emit q->objectRemoved(index, obj);
                 if (obj)
                     m_instanceModel->release(obj);
             }
@@ -201,7 +203,8 @@ void Quick3DNodeInstantiatorPrivate::_q_modelUpdated(const QQmlChangeSet &change
             m_objects = m_objects.mid(0, index) + movedObjects + m_objects.mid(index);
         } else for (int i = 0; i < insert.count; ++i) {
             int modelIndex = index + i;
-            QObject *obj = m_instanceModel->object(modelIndex, m_async);
+            QObject *obj = m_instanceModel->object(modelIndex, m_async ?
+                QQmlIncubator::Asynchronous : QQmlIncubator::AsynchronousIfNested);
             if (obj)
                 _q_createdItem(modelIndex, obj);
         }
@@ -209,7 +212,7 @@ void Quick3DNodeInstantiatorPrivate::_q_modelUpdated(const QQmlChangeSet &change
     }
 
     if (difference != 0)
-        q->countChanged();
+        emit q->countChanged();
 }
 
 void Quick3DNodeInstantiatorPrivate::makeModel()
@@ -244,6 +247,7 @@ void Quick3DNodeInstantiatorPrivate::makeModel()
 Quick3DNodeInstantiator::Quick3DNodeInstantiator(QNode *parent)
     : QNode(*new Quick3DNodeInstantiatorPrivate, parent)
 {
+    connect(this, &QNode::parentChanged, this, &Quick3DNodeInstantiator::onParentChanged);
 }
 
 /*!
@@ -500,6 +504,17 @@ void Quick3DNodeInstantiator::componentComplete()
     }
 }
 
+/*!
+ \internal
+*/
+void Quick3DNodeInstantiator::onParentChanged(QObject *parent)
+{
+    Q_D(const Quick3DNodeInstantiator);
+    auto parentNode = static_cast<QNode *>(parent);
+    for (auto obj : d->m_objects)
+        static_cast<QNode *>(obj.data())->setParent(parentNode);
+}
+
 // TODO: Avoid cloning here
 //void Quick3DNodeInstantiator::copy(const QNode *ref)
 //{
