@@ -38,10 +38,11 @@
 ****************************************************************************/
 
 #include "qqmlaspectengine_p.h"
-#include <QQmlComponent>
-#include <QQmlContext>
-#include <QDebug>
+
 #include <Qt3DCore/qentity.h>
+#include <QtCore/QDebug>
+#include <QtQml/QQmlComponent>
+#include <QtQml/QQmlContext>
 
 QT_BEGIN_NAMESPACE
 
@@ -65,6 +66,7 @@ QQmlAspectEnginePrivate::QQmlAspectEnginePrivate()
 
 /*!
  * \class Qt3DCore::Quick::QQmlAspectEngine
+ * \inheaderfile Qt3DQuick/QQmlAspectEngine
  * \inmodule Qt3DCore
  *
  * \brief The QQmlAspectEngine provides an environment for the QAspectEngine and
