@@ -54,6 +54,7 @@ namespace Render {
 
 CameraLens::CameraLens()
     : BackendNode()
+    , m_exposure(0.0f)
 {
 }
 
@@ -67,11 +68,24 @@ void CameraLens::cleanup()
     QBackendNode::setEnabled(false);
 }
 
+QMatrix4x4 CameraLens::viewMatrix(const QMatrix4x4 &worldTransform)
+{
+    const QVector4D position = worldTransform * QVector4D(0.0f, 0.0f, 0.0f, 1.0f);
+    // OpenGL convention is looking down -Z
+    const QVector4D viewDirection = worldTransform * QVector4D(0.0f, 0.0f, -1.0f, 0.0f);
+    const QVector4D upVector = worldTransform * QVector4D(0.0f, 1.0f, 0.0f, 0.0f);
+
+    QMatrix4x4 m;
+    m.lookAt(position.toVector3D(), (position + viewDirection).toVector3D(), upVector.toVector3D());
+    return m;
+}
+
 void CameraLens::initializeFromPeer(const Qt3DCore::QNodeCreatedChangeBasePtr &change)
 {
     const auto typedChange = qSharedPointerCast<Qt3DCore::QNodeCreatedChange<QCameraLensData>>(change);
     const auto &data = typedChange->data;
     m_projection = data.projectionMatrix;
+    m_exposure = data.exposure;
 }
 
 void CameraLens::setProjection(const QMatrix4x4 &projection)
@@ -79,6 +93,11 @@ void CameraLens::setProjection(const QMatrix4x4 &projection)
     m_projection = projection;
 }
 
+void CameraLens::setExposure(float exposure)
+{
+    m_exposure = exposure;
+}
+
 void CameraLens::sceneChangeEvent(const Qt3DCore::QSceneChangePtr &e)
 {
     switch (e->type()) {
@@ -88,6 +107,8 @@ void CameraLens::sceneChangeEvent(const Qt3DCore::QSceneChangePtr &e)
         if (propertyChange->propertyName() == QByteArrayLiteral("projectionMatrix")) {
             QMatrix4x4 projectionMatrix = propertyChange->value().value<QMatrix4x4>();
             m_projection = projectionMatrix;
+        } else if (propertyChange->propertyName() == QByteArrayLiteral("exposure")) {
+            setExposure(propertyChange->value().toFloat());
         }
 
         markDirty(AbstractRenderer::AllDirty);
