@@ -62,11 +62,21 @@ QT_BEGIN_NAMESPACE
 #define GL_SAMPLER_2D_ARRAY_SHADOW        0x8DC4
 #endif
 
+// ES 2.0 FBO
+#ifndef GL_DRAW_FRAMEBUFFER
+#define GL_DRAW_FRAMEBUFFER               0x8CA9
+#endif
+
+#ifndef GL_READ_FRAMEBUFFER
+#define GL_READ_FRAMEBUFFER               0x8CA8
+#endif
+
 namespace Qt3DRender {
 namespace Render {
 
-GraphicsHelperES2::GraphicsHelperES2() :
-    m_funcs(0)
+GraphicsHelperES2::GraphicsHelperES2()
+    : m_funcs(0)
+    , m_supportFramebufferBlit(false)
 {
 }
 
@@ -80,6 +90,9 @@ void GraphicsHelperES2::initializeHelper(QOpenGLContext *context,
     Q_ASSERT(context);
     m_funcs = context->functions();
     Q_ASSERT(m_funcs);
+    m_ext.reset(new QOpenGLExtensions(context));
+    if (m_ext->hasOpenGLExtension(QOpenGLExtensions::FramebufferBlit))
+        m_supportFramebufferBlit = true;
 }
 
 void GraphicsHelperES2::drawElementsInstancedBaseVertexBaseInstance(GLenum primitiveType,
@@ -156,6 +169,16 @@ void GraphicsHelperES2::drawArrays(GLenum primitiveType,
                           count);
 }
 
+void GraphicsHelperES2::drawElementsIndirect(GLenum, GLenum, void *)
+{
+    qWarning() << "Indirect Drawing is not supported with OpenGL ES 2";
+}
+
+void GraphicsHelperES2::drawArraysIndirect(GLenum , void *)
+{
+    qWarning() << "Indirect Drawing is not supported with OpenGL ES 2";
+}
+
 void GraphicsHelperES2::setVerticesPerPatch(GLint verticesPerPatch)
 {
     Q_UNUSED(verticesPerPatch);
@@ -185,6 +208,9 @@ QVector<ShaderUniform> GraphicsHelperES2::programUniformsAndLocations(GLuint pro
         uniformName[sizeof(uniformName) - 1] = '\0';
         uniform.m_location = m_funcs->glGetUniformLocation(programId, uniformName);
         uniform.m_name = QString::fromUtf8(uniformName, uniformNameLength);
+        // Work around for uniform array names that aren't returned with [0] by some drivers
+        if (uniform.m_size > 1 && !uniform.m_name.endsWith(QLatin1String("[0]")))
+            uniform.m_name.append(QLatin1String("[0]"));
         uniforms.append(uniform);
     }
     return uniforms;
@@ -304,9 +330,20 @@ void GraphicsHelperES2::releaseFrameBufferObject(GLuint frameBufferId)
     m_funcs->glDeleteFramebuffers(1, &frameBufferId);
 }
 
-void GraphicsHelperES2::bindFrameBufferObject(GLuint frameBufferId)
+void GraphicsHelperES2::bindFrameBufferObject(GLuint frameBufferId, FBOBindMode mode)
 {
-    m_funcs->glBindFramebuffer(GL_FRAMEBUFFER, frameBufferId);
+    switch (mode) {
+    case FBODraw:
+        m_funcs->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, frameBufferId);
+        return;
+    case FBORead:
+        m_funcs->glBindFramebuffer(GL_READ_FRAMEBUFFER, frameBufferId);
+        return;
+    case FBOReadAndDraw:
+    default:
+        m_funcs->glBindFramebuffer(GL_FRAMEBUFFER, frameBufferId);
+        return;
+    }
 }
 
 GLuint GraphicsHelperES2::boundFrameBufferObject()
@@ -356,6 +393,8 @@ bool GraphicsHelperES2::supportsFeature(GraphicsHelperInterface::Feature feature
     switch (feature) {
     case RenderBufferDimensionRetrieval:
         return true;
+    case BlitFramebuffer:
+        return m_supportFramebufferBlit;
     default:
         return false;
     }
@@ -482,6 +521,12 @@ GLint GraphicsHelperES2::maxClipPlaneCount()
     return 0;
 }
 
+void GraphicsHelperES2::memoryBarrier(QMemoryBarrier::Operations barriers)
+{
+    Q_UNUSED(barriers);
+    qWarning() << "memory barrier is not supported by OpenGL ES 2.0 (since 4.3)";
+}
+
 void GraphicsHelperES2::enablePrimitiveRestart(int)
 {
 }
@@ -559,6 +604,20 @@ void GraphicsHelperES2::dispatchCompute(GLuint wx, GLuint wy, GLuint wz)
     qWarning() << "Compute Shaders are not supported by ES 2.0 (since ES 3.1)";
 }
 
+char *GraphicsHelperES2::mapBuffer(GLenum target)
+{
+    Q_UNUSED(target);
+    qWarning() << "Map buffer is not a core requirement for ES 2.0";
+    return nullptr;
+}
+
+GLboolean GraphicsHelperES2::unmapBuffer(GLenum target)
+{
+    Q_UNUSED(target);
+    qWarning() << "unMap buffer is not a core requirement for ES 2.0";
+    return false;
+}
+
 void GraphicsHelperES2::glUniform1fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniform1fv(location, count, values);
@@ -709,17 +768,10 @@ UniformType GraphicsHelperES2::uniformTypeFromGLType(GLenum type)
 
 void GraphicsHelperES2::blitFramebuffer(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
 {
-    Q_UNUSED(srcX0);
-    Q_UNUSED(srcX1);
-    Q_UNUSED(srcY0);
-    Q_UNUSED(srcY1);
-    Q_UNUSED(dstX0);
-    Q_UNUSED(dstX1);
-    Q_UNUSED(dstY0);
-    Q_UNUSED(dstY1);
-    Q_UNUSED(mask);
-    Q_UNUSED(filter);
-    qWarning() << "Framebuffer blits are not supported by ES 2.0 (since ES 3.1)";
+    if (!m_supportFramebufferBlit)
+        qWarning() << "Framebuffer blits are not supported by ES 2.0 (since ES 3.1)";
+    else
+        m_ext->glBlitFramebuffer(srcX0, srcY0, srcX1, srcY1, dstX0, dstY0, dstX1, dstY1, mask, filter);
 }
 
 } // namespace Render
