@@ -58,7 +58,7 @@ GraphicsHelperGL2::GraphicsHelperGL2()
 }
 
 void GraphicsHelperGL2::initializeHelper(QOpenGLContext *context,
-                                          QAbstractOpenGLFunctions *functions)
+                                         QAbstractOpenGLFunctions *functions)
 {
     Q_UNUSED(context);
     m_funcs = static_cast<QOpenGLFunctions_2_0*>(functions);
@@ -95,9 +95,9 @@ void GraphicsHelperGL2::drawElementsInstancedBaseVertexBaseInstance(GLenum primi
 }
 
 void GraphicsHelperGL2::drawArraysInstanced(GLenum primitiveType,
-                                             GLint first,
-                                             GLsizei count,
-                                             GLsizei instances)
+                                            GLint first,
+                                            GLsizei count,
+                                            GLsizei instances)
 {
     for (GLint i = 0; i < instances; i++)
         drawArrays(primitiveType,
@@ -116,10 +116,10 @@ void GraphicsHelperGL2::drawArraysInstancedBaseInstance(GLenum primitiveType, GL
 }
 
 void GraphicsHelperGL2::drawElements(GLenum primitiveType,
-                                      GLsizei primitiveCount,
-                                      GLint indexType,
-                                      void *indices,
-                                      GLint baseVertex)
+                                     GLsizei primitiveCount,
+                                     GLint indexType,
+                                     void *indices,
+                                     GLint baseVertex)
 {
     if (baseVertex != 0)
         qWarning() << "glDrawElementsBaseVertex is not supported with OpenGL 2";
@@ -131,14 +131,24 @@ void GraphicsHelperGL2::drawElements(GLenum primitiveType,
 }
 
 void GraphicsHelperGL2::drawArrays(GLenum primitiveType,
-                                    GLint first,
-                                    GLsizei count)
+                                   GLint first,
+                                   GLsizei count)
 {
     m_funcs->glDrawArrays(primitiveType,
                           first,
                           count);
 }
 
+void GraphicsHelperGL2::drawElementsIndirect(GLenum, GLenum, void *)
+{
+    qWarning() << "Indirect Drawing is not supported with OpenGL 2";
+}
+
+void GraphicsHelperGL2::drawArraysIndirect(GLenum , void *)
+{
+    qWarning() << "Indirect Drawing is not supported with OpenGL 2";
+}
+
 void GraphicsHelperGL2::setVerticesPerPatch(GLint verticesPerPatch)
 {
     Q_UNUSED(verticesPerPatch);
@@ -168,6 +178,10 @@ QVector<ShaderUniform> GraphicsHelperGL2::programUniformsAndLocations(GLuint pro
         uniformName[sizeof(uniformName) - 1] = '\0';
         uniform.m_location = m_funcs->glGetUniformLocation(programId, uniformName);
         uniform.m_name = QString::fromUtf8(uniformName, uniformNameLength);
+        // Work around for uniform array names that aren't returned with [0] by some drivers
+        if (uniform.m_size > 1 && !uniform.m_name.endsWith(QLatin1String("[0]")))
+            uniform.m_name.append(QLatin1String("[0]"));
+        uniform.m_rawByteSize = uniformByteSize(uniform);
         uniforms.append(uniform);
     }
     return uniforms;
@@ -211,7 +225,7 @@ QVector<ShaderStorageBlock> GraphicsHelperGL2::programShaderStorageBlocks(GLuint
 }
 
 void GraphicsHelperGL2::vertexAttribDivisor(GLuint index,
-                                             GLuint divisor)
+                                            GLuint divisor)
 {
     Q_UNUSED(index);
     Q_UNUSED(divisor);
@@ -316,8 +330,14 @@ void GraphicsHelperGL2::bindFrameBufferAttachment(QOpenGLTexture *texture, const
         else
             qCritical() << "DepthStencil Attachment not supported on OpenGL 2.0";
 
+        const QOpenGLTexture::Target target = texture->target();
+
+        if (target == QOpenGLTexture::TargetCubeMap && attachment.m_face == QAbstractTexture::AllFaces) {
+            qWarning() << "OpenGL 2.0 doesn't handle attaching all the faces of a cube map texture at once to an FBO";
+            return;
+        }
+
         texture->bind();
-        QOpenGLTexture::Target target = texture->target();
         if (target == QOpenGLTexture::Target3D)
             m_fboFuncs->glFramebufferTexture3D(GL_DRAW_FRAMEBUFFER, attr, target, texture->textureId(), attachment.m_mipLevel, attachment.m_layer);
         else if (target == QOpenGLTexture::TargetCubeMap)
@@ -358,104 +378,24 @@ void GraphicsHelperGL2::bindFragDataLocation(GLuint, const QHash<QString, int> &
     qCritical() << "bindFragDataLocation is not supported by GL 2.0";
 }
 
-void GraphicsHelperGL2::bindUniform(const QVariant &v, const ShaderUniform &description)
+void GraphicsHelperGL2::bindFrameBufferObject(GLuint frameBufferId, FBOBindMode mode)
 {
-    switch (description.m_type) {
-
-    case GL_FLOAT:
-        m_funcs->glUniform1fv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLfloat>(v, description.m_size, 1));
-        break;
-
-    case GL_FLOAT_VEC2:
-        m_funcs->glUniform2fv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLfloat>(v, description.m_size, 2));
-        break;
-
-    case GL_FLOAT_VEC3:
-        m_funcs->glUniform3fv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLfloat>(v, description.m_size, 3));
-        break;
-
-    case GL_FLOAT_VEC4:
-        m_funcs->glUniform4fv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLfloat>(v, description.m_size, 4));
-        break;
-
-    case GL_FLOAT_MAT2:
-        m_funcs->glUniformMatrix2fv(description.m_location, description.m_size, GL_FALSE,
-                                    QGraphicsUtils::valueArrayFromVariant<GLfloat>(v, description.m_size, 4));
-        break;
-
-    case GL_FLOAT_MAT3:
-        m_funcs->glUniformMatrix3fv(description.m_location, description.m_size, GL_FALSE,
-                                    QGraphicsUtils::valueArrayFromVariant<GLfloat>(v, description.m_size, 9));
-        break;
-
-    case GL_FLOAT_MAT4:
-        m_funcs->glUniformMatrix4fv(description.m_location, description.m_size, GL_FALSE,
-                                    QGraphicsUtils::valueArrayFromVariant<GLfloat>(v, description.m_size, 16));
-        break;
-
-    case GL_INT:
-        m_funcs->glUniform1iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 1));
-        break;
-
-    case GL_INT_VEC2:
-        m_funcs->glUniform2iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 2));
-        break;
-
-    case GL_INT_VEC3:
-        m_funcs->glUniform3iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 3));
-        break;
-
-    case GL_INT_VEC4:
-        m_funcs->glUniform4iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 4));
-        break;
-
-    case GL_BOOL:
-        m_funcs->glUniform1iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 1));
-        break;
-
-    case GL_BOOL_VEC2:
-        m_funcs->glUniform2iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 2));
-        break;
-
-    case GL_BOOL_VEC3:
-        m_funcs->glUniform3iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 3));
-        break;
-
-    case GL_BOOL_VEC4:
-        m_funcs->glUniform4iv(description.m_location, description.m_size,
-                              QGraphicsUtils::valueArrayFromVariant<GLint>(v, description.m_size, 4));
-        break;
-
-    case GL_SAMPLER_2D:
-    case GL_SAMPLER_CUBE: {
-        Q_ASSERT(description.m_size == 1);
-        m_funcs->glUniform1i(description.m_location, v.toInt());
-        break;
-    }
-
-    default:
-        qWarning() << Q_FUNC_INFO << "unsupported uniform type:" << description.m_type << "for " << description.m_name;
-        break;
-    }
-}
-
-void GraphicsHelperGL2::bindFrameBufferObject(GLuint frameBufferId)
-{
-    if (m_fboFuncs != nullptr)
-        m_fboFuncs->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, frameBufferId);
-    else
+    if (m_fboFuncs != nullptr) {
+        switch (mode) {
+        case FBODraw:
+            m_fboFuncs->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, frameBufferId);
+            return;
+        case FBORead:
+            m_fboFuncs->glBindFramebuffer(GL_READ_FRAMEBUFFER, frameBufferId);
+            return;
+        case FBOReadAndDraw:
+        default:
+            m_fboFuncs->glBindFramebuffer(GL_FRAMEBUFFER, frameBufferId);
+            return;
+        }
+    } else {
         qWarning() << "FBO not supported by your OpenGL hardware";
+    }
 }
 
 GLuint GraphicsHelperGL2::boundFrameBufferObject()
@@ -524,14 +464,38 @@ uint GraphicsHelperGL2::uniformByteSize(const ShaderUniform &description)
         rawByteSize = matrixStride ? 2 * matrixStride : 16;
         break;
 
+    case GL_FLOAT_MAT2x4:
+        rawByteSize = matrixStride ? 2 * matrixStride : 32;
+        break;
+
+    case GL_FLOAT_MAT4x2:
+        rawByteSize = matrixStride ? 4 * matrixStride : 32;
+        break;
+
     case GL_FLOAT_MAT3:
         rawByteSize = matrixStride ? 3 * matrixStride : 36;
         break;
 
+    case GL_FLOAT_MAT2x3:
+        rawByteSize = matrixStride ? 2 * matrixStride : 24;
+        break;
+
+    case GL_FLOAT_MAT3x2:
+        rawByteSize = matrixStride ? 3 * matrixStride : 24;
+        break;
+
     case GL_FLOAT_MAT4:
         rawByteSize = matrixStride ? 4 * matrixStride : 64;
         break;
 
+    case GL_FLOAT_MAT4x3:
+        rawByteSize = matrixStride ? 4 * matrixStride : 48;
+        break;
+
+    case GL_FLOAT_MAT3x4:
+        rawByteSize = matrixStride ? 3 * matrixStride : 48;
+        break;
+
     case GL_BOOL:
         rawByteSize = 1;
         break;
@@ -550,10 +514,17 @@ uint GraphicsHelperGL2::uniformByteSize(const ShaderUniform &description)
 
     case GL_INT:
     case GL_FLOAT:
+    case GL_SAMPLER_1D:
+    case GL_SAMPLER_1D_SHADOW:
     case GL_SAMPLER_2D:
+    case GL_SAMPLER_2D_SHADOW:
+    case GL_SAMPLER_3D:
     case GL_SAMPLER_CUBE:
         rawByteSize = 4;
         break;
+
+    default:
+        Q_UNREACHABLE();
     }
 
     return arrayStride ? rawByteSize * arrayStride : rawByteSize;
@@ -587,6 +558,12 @@ GLint GraphicsHelperGL2::maxClipPlaneCount()
     return max;
 }
 
+void GraphicsHelperGL2::memoryBarrier(QMemoryBarrier::Operations barriers)
+{
+    Q_UNUSED(barriers);
+    qWarning() << "memory barrier is not supported by OpenGL 2.0 (since 4.3)";
+}
+
 void GraphicsHelperGL2::enablePrimitiveRestart(int)
 {
 }
@@ -604,15 +581,11 @@ void GraphicsHelperGL2::clearBufferf(GLint drawbuffer, const QVector4D &values)
 
 void GraphicsHelperGL2::pointSize(bool programmable, GLfloat value)
 {
-    // Print a warning once for trying to set GL_PROGRAM_POINT_SIZE
-    if (programmable) {
-        static bool warned = false;
-        if (!warned) {
-            qWarning() << "GL_PROGRAM_POINT_SIZE is not supported by OpenGL 2.0 (since 3.2)";
-            warned = true;
-        }
-    }
-    m_funcs->glPointSize(value);
+    m_funcs->glEnable(GL_POINT_SPRITE);
+    if (programmable)
+        m_funcs->glEnable(GL_VERTEX_PROGRAM_POINT_SIZE);
+    else
+        m_funcs->glPointSize(value);
 }
 
 void GraphicsHelperGL2::enablei(GLenum cap, GLuint index)
@@ -663,6 +636,184 @@ void GraphicsHelperGL2::dispatchCompute(GLuint wx, GLuint wy, GLuint wz)
     qWarning() << "Compute Shaders are not supported by OpenGL 2.0 (since OpenGL 4.3)";
 }
 
+char *GraphicsHelperGL2::mapBuffer(GLenum target)
+{
+    return static_cast<char*>(m_funcs->glMapBuffer(target, GL_READ_WRITE));
+}
+
+GLboolean GraphicsHelperGL2::unmapBuffer(GLenum target)
+{
+    return m_funcs->glUnmapBuffer(target);
+}
+
+void GraphicsHelperGL2::glUniform1fv(GLint location, GLsizei count, const GLfloat *values)
+{
+    m_funcs->glUniform1fv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform2fv(GLint location, GLsizei count, const GLfloat *values)
+{
+    m_funcs->glUniform2fv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform3fv(GLint location, GLsizei count, const GLfloat *values)
+{
+    m_funcs->glUniform3fv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform4fv(GLint location, GLsizei count, const GLfloat *values)
+{
+    m_funcs->glUniform4fv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform1iv(GLint location, GLsizei count, const GLint *values)
+{
+    m_funcs->glUniform1iv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform2iv(GLint location, GLsizei count, const GLint *values)
+{
+    m_funcs->glUniform2iv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform3iv(GLint location, GLsizei count, const GLint *values)
+{
+    m_funcs->glUniform3iv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform4iv(GLint location, GLsizei count, const GLint *values)
+{
+    m_funcs->glUniform4iv(location, count, values);
+}
+
+void GraphicsHelperGL2::glUniform1uiv(GLint , GLsizei , const GLuint *)
+{
+    qWarning() << "glUniform1uiv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniform2uiv(GLint , GLsizei , const GLuint *)
+{
+    qWarning() << "glUniform2uiv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniform3uiv(GLint , GLsizei , const GLuint *)
+{
+    qWarning() << "glUniform3uiv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniform4uiv(GLint , GLsizei , const GLuint *)
+{
+    qWarning() << "glUniform4uiv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniformMatrix2fv(GLint location, GLsizei count, const GLfloat *values)
+{
+    m_funcs->glUniformMatrix2fv(location, count, false, values);
+}
+
+void GraphicsHelperGL2::glUniformMatrix3fv(GLint location, GLsizei count, const GLfloat *values)
+{
+    m_funcs->glUniformMatrix3fv(location, count, false, values);
+}
+
+void GraphicsHelperGL2::glUniformMatrix4fv(GLint location, GLsizei count, const GLfloat *values)
+{
+    m_funcs->glUniformMatrix4fv(location, count, false, values);
+}
+
+void GraphicsHelperGL2::glUniformMatrix2x3fv(GLint , GLsizei , const GLfloat *)
+{
+    qWarning() << "glUniformMatrix2x3fv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniformMatrix3x2fv(GLint , GLsizei , const GLfloat *)
+{
+    qWarning() << "glUniformMatrix3x2fv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniformMatrix2x4fv(GLint , GLsizei , const GLfloat *)
+{
+    qWarning() << "glUniformMatrix2x4fv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniformMatrix4x2fv(GLint , GLsizei , const GLfloat *)
+{
+    qWarning() << "glUniformMatrix4x2fv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniformMatrix3x4fv(GLint , GLsizei , const GLfloat *)
+{
+    qWarning() << "glUniformMatrix3x4fv not supported by GL 2";
+}
+
+void GraphicsHelperGL2::glUniformMatrix4x3fv(GLint , GLsizei , const GLfloat *)
+{
+    qWarning() << "glUniformMatrix4x3fv not supported by GL 2";
+}
+
+UniformType GraphicsHelperGL2::uniformTypeFromGLType(GLenum type)
+{
+    switch (type) {
+    case GL_FLOAT:
+        return UniformType::Float;
+    case GL_FLOAT_VEC2:
+        return UniformType::Vec2;
+    case GL_FLOAT_VEC3:
+        return UniformType::Vec3;
+    case GL_FLOAT_VEC4:
+        return UniformType::Vec4;
+    case GL_FLOAT_MAT2:
+        return UniformType::Mat2;
+    case GL_FLOAT_MAT3:
+        return UniformType::Mat3;
+    case GL_FLOAT_MAT4:
+        return UniformType::Mat4;
+    case GL_INT:
+        return UniformType::Int;
+    case GL_INT_VEC2:
+        return UniformType::IVec2;
+    case GL_INT_VEC3:
+        return UniformType::IVec3;
+    case GL_INT_VEC4:
+        return UniformType::IVec4;
+    case GL_BOOL:
+        return UniformType::Bool;
+    case GL_BOOL_VEC2:
+        return UniformType::BVec2;
+    case GL_BOOL_VEC3:
+        return UniformType::BVec3;
+    case GL_BOOL_VEC4:
+        return UniformType::BVec4;
+
+    case GL_SAMPLER_1D:
+    case GL_SAMPLER_1D_SHADOW:
+    case GL_SAMPLER_2D:
+    case GL_SAMPLER_2D_SHADOW:
+    case GL_SAMPLER_CUBE:
+    case GL_SAMPLER_3D:
+        return UniformType::Sampler;
+
+    default:
+        Q_UNREACHABLE();
+        return UniformType::Float;
+    }
+}
+
+void GraphicsHelperGL2::blitFramebuffer(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
+{
+    Q_UNUSED(srcX0);
+    Q_UNUSED(srcX1);
+    Q_UNUSED(srcY0);
+    Q_UNUSED(srcY1);
+    Q_UNUSED(dstX0);
+    Q_UNUSED(dstX1);
+    Q_UNUSED(dstY0);
+    Q_UNUSED(dstY1);
+    Q_UNUSED(mask);
+    Q_UNUSED(filter);
+    qWarning() << "Framebuffer blits are not supported by ES 2.0 (since ES 3.1)";
+}
+
 } // namespace Render
 } // namespace Qt3DRender
 
