@@ -46,8 +46,6 @@ namespace Render {
 
 SceneManager::SceneManager() : Qt3DCore::QResourceManager<Scene,
                                                 Qt3DCore::QNodeId,
-                                                8,
-                                                Qt3DCore::ArrayAllocatingPolicy,
                                                 Qt3DCore::ObjectLevelLockingPolicy>()
 {
 }
