@@ -49,15 +49,23 @@
 #include <Qt3DRender/private/rendersettings_p.h>
 #include <Qt3DRender/qgeometryrenderer.h>
 #include <Qt3DRender/private/job_common_p.h>
+#include <Qt3DRender/private/qpickevent_p.h>
+
+#include <QSurface>
+#include <QWindow>
+#include <QOffscreenSurface>
 
 QT_BEGIN_NAMESPACE
 
 namespace Qt3DRender {
+using namespace Qt3DRender::RayCasting;
 
 namespace Render {
 
 namespace {
 
+typedef PickingUtils::AbstractCollisionGathererFunctor::result_type HitList;
+
 void setEventButtonAndModifiers(const QMouseEvent &event, QPickEvent::Buttons &eventButton, int &eventButtons, int &eventModifiers)
 {
     switch (event.button()) {
@@ -104,6 +112,7 @@ PickBoundingVolumeJob::PickBoundingVolumeJob()
     , m_node(nullptr)
     , m_frameGraphRoot(nullptr)
     , m_renderSettings(nullptr)
+    , m_pickersDirty(true)
 {
     SET_JOB_RUN_STAT_TYPE(this, JobTypes::PickBoundingVolume, 0);
 }
@@ -113,11 +122,16 @@ void PickBoundingVolumeJob::setRoot(Entity *root)
     m_node = root;
 }
 
-void PickBoundingVolumeJob::setMouseEvents(const QList<QMouseEvent> &pendingEvents)
+void PickBoundingVolumeJob::setMouseEvents(const QList<QPair<QObject*, QMouseEvent>> &pendingEvents)
 {
     m_pendingMouseEvents = pendingEvents;
 }
 
+void PickBoundingVolumeJob::setKeyEvents(const QList<QKeyEvent> &pendingEvents)
+{
+    m_pendingKeyEvents = pendingEvents;
+}
+
 void PickBoundingVolumeJob::setFrameGraphRoot(FrameGraphNode *frameGraphRoot)
 {
     m_frameGraphRoot = frameGraphRoot;
@@ -128,16 +142,17 @@ void PickBoundingVolumeJob::setRenderSettings(RenderSettings *settings)
     m_renderSettings = settings;
 }
 
-QRay3D PickBoundingVolumeJob::intersectionRay(const QPoint &pos, const QMatrix4x4 &viewMatrix, const QMatrix4x4 &projectionMatrix, const QRect &viewport)
+RayCasting::QRay3D PickBoundingVolumeJob::intersectionRay(const QPoint &pos, const QMatrix4x4 &viewMatrix,
+                                                          const QMatrix4x4 &projectionMatrix, const QRect &viewport)
 {
     QVector3D nearPos = QVector3D(pos.x(), pos.y(), 0.0f);
     nearPos = nearPos.unproject(viewMatrix, projectionMatrix, viewport);
     QVector3D farPos = QVector3D(pos.x(), pos.y(), 1.0f);
     farPos = farPos.unproject(viewMatrix, projectionMatrix, viewport);
 
-    return QRay3D(nearPos,
-                  (farPos - nearPos).normalized(),
-                  (farPos - nearPos).length());
+    return RayCasting::QRay3D(nearPos,
+                              (farPos - nearPos).normalized(),
+                              (farPos - nearPos).length());
 }
 
 bool PickBoundingVolumeJob::runHelper()
@@ -150,61 +165,119 @@ bool PickBoundingVolumeJob::runHelper()
     if (mouseEvents.empty())
         return false;
 
+    // Quickly look which picker settings we've got
+    if (m_pickersDirty) {
+        m_pickersDirty = false;
+        m_oneEnabledAtLeast = false;
+        m_oneHoverAtLeast = false;
+
+        const auto activeHandles = m_manager->objectPickerManager()->activeHandles();
+        for (auto handle : activeHandles) {
+            auto picker = m_manager->objectPickerManager()->data(handle);
+            m_oneEnabledAtLeast |= picker->isEnabled();
+            m_oneHoverAtLeast |= picker->isHoverEnabled();
+            if (m_oneEnabledAtLeast && m_oneHoverAtLeast)
+                break;
+        }
+    }
+
+    // bail out early if no picker is enabled
+    if (!m_oneEnabledAtLeast)
+        return false;
+
+    bool hasMoveEvent = false;
+    bool hasOtherEvent = false;
+    // Quickly look which types of events we've got
+    for (const auto &event : mouseEvents) {
+        const bool isMove = (event.second.type() == QEvent::MouseMove);
+        hasMoveEvent |= isMove;
+        hasOtherEvent |= !isMove;
+    }
+
+    // In the case we have a move event, find if we actually have
+    // an object picker that cares about these
+    if (!hasOtherEvent) {
+        // Retrieve the last used object picker
+        ObjectPicker *lastCurrentPicker = m_manager->objectPickerManager()->data(m_currentPicker);
+
+        // The only way to set lastCurrentPicker is to click
+        // so we can return since if we're there it means we
+        // have only move events. But keep on if hover support
+        // is needed
+        if (lastCurrentPicker == nullptr && !m_oneHoverAtLeast)
+            return false;
+
+        const bool caresAboutMove = (hasMoveEvent &&
+                                      (m_oneHoverAtLeast ||
+                                        (lastCurrentPicker && lastCurrentPicker->isDragEnabled())));
+        // Early return if the current object picker doesn't care about move events
+        if (!caresAboutMove)
+            return false;
+    }
+
     PickingUtils::ViewportCameraAreaGatherer vcaGatherer;
     // TO DO: We could cache this and only gather when we know the FrameGraph tree has changed
-    const QVector<PickingUtils::ViewportCameraAreaTriplet> vcaTriplets = vcaGatherer.gather(m_frameGraphRoot);
+    const QVector<PickingUtils::ViewportCameraAreaDetails> vcaDetails = vcaGatherer.gather(m_frameGraphRoot);
 
     // If we have no viewport / camera or area, return early
-    if (vcaTriplets.empty())
+    if (vcaDetails.empty())
         return false;
 
     // TO DO:
     // If we have move or hover move events that someone cares about, we try to avoid expensive computations
     // by compressing them into a single one
 
-    // Gather the entities for the frame
-    // TO DO: We could skip doing that every frame and only do it when we know for sure
-    // that the tree structure has changed
-    PickingUtils::EntityGatherer entitiesGatherer(m_node);
-
     // Store the reducer function which varies depending on the picking settings set on the renderer
     using ReducerFunction = PickingUtils::CollisionVisitor::HitList (*)(PickingUtils::CollisionVisitor::HitList &results, const PickingUtils::CollisionVisitor::HitList &intermediate);
 
     const bool trianglePickingRequested = (m_renderSettings->pickMethod() == QPickingSettings::TrianglePicking);
     const bool allHitsRequested = (m_renderSettings->pickResultMode() == QPickingSettings::AllPicks);
+    const bool frontFaceRequested =
+            m_renderSettings->faceOrientationPickingMode() != QPickingSettings::BackFace;
+    const bool backFaceRequested =
+            m_renderSettings->faceOrientationPickingMode() != QPickingSettings::FrontFace;
 
     // Select the best reduction function based on the settings
     const ReducerFunction reducerOp = allHitsRequested ? PickingUtils::reduceToAllHits : PickingUtils::reduceToFirstHit;
 
     // For each mouse event
-    for (const QMouseEvent &event : mouseEvents) {
+    for (const auto &event : mouseEvents) {
         m_hoveredPickersToClear = m_hoveredPickers;
 
         QPickEvent::Buttons eventButton = QPickEvent::NoButton;
         int eventButtons = 0;
         int eventModifiers = QPickEvent::NoModifier;
 
-        setEventButtonAndModifiers(event, eventButton, eventButtons, eventModifiers);
+        setEventButtonAndModifiers(event.second, eventButton, eventButtons, eventModifiers);
 
-        // For each triplet of Viewport / Camera and Area
-        for (const PickingUtils::ViewportCameraAreaTriplet &vca : vcaTriplets) {
-            typedef PickingUtils::AbstractCollisionGathererFunctor::result_type HitList;
+        // For each Viewport / Camera and Area entry
+        for (const PickingUtils::ViewportCameraAreaDetails &vca : vcaDetails) {
             HitList sphereHits;
-            QRay3D ray = rayForViewportAndCamera(vca.area, event.pos(), vca.viewport, vca.cameraId);
-            if (trianglePickingRequested) {
-                PickingUtils::TriangleCollisionGathererFunctor gathererFunctor;
-                gathererFunctor.m_manager = m_manager;
-                gathererFunctor.m_ray = ray;
-                sphereHits = QtConcurrent::blockingMappedReduced<HitList>(entitiesGatherer.entities(), gathererFunctor, reducerOp);
-            } else {
-                PickingUtils::EntityCollisionGathererFunctor gathererFunctor;
-                gathererFunctor.m_manager = m_manager;
-                gathererFunctor.m_ray = ray;
-                sphereHits = QtConcurrent::blockingMappedReduced<HitList>(entitiesGatherer.entities(), gathererFunctor, reducerOp);
+            QRay3D ray = rayForViewportAndCamera(vca, event.first, event.second);
+            if (!ray.isValid())
+                continue;
+
+            PickingUtils::HierarchicalEntityPicker entityPicker(ray);
+            if (entityPicker.collectHits(m_node)) {
+                if (trianglePickingRequested) {
+                    PickingUtils::TriangleCollisionGathererFunctor gathererFunctor;
+                    gathererFunctor.m_frontFaceRequested = frontFaceRequested;
+                    gathererFunctor.m_backFaceRequested = backFaceRequested;
+                    gathererFunctor.m_manager = m_manager;
+                    gathererFunctor.m_ray = ray;
+                    sphereHits = QtConcurrent::blockingMappedReduced<HitList>(entityPicker.entities(),
+                                                                              gathererFunctor, reducerOp);
+                } else {
+                    sphereHits = entityPicker.hits();
+                    PickingUtils::AbstractCollisionGathererFunctor::sortHits(sphereHits);
+                    if (!allHitsRequested)
+                        sphereHits = { sphereHits.front() };
+                }
             }
 
             // Dispatch events based on hit results
-            dispatchPickEvents(event, sphereHits, eventButton, eventButtons, eventModifiers);
+            dispatchPickEvents(event.second, sphereHits, eventButton, eventButtons, eventModifiers,
+                               trianglePickingRequested, allHitsRequested);
         }
     }
 
@@ -221,6 +294,11 @@ void PickBoundingVolumeJob::setManagers(NodeManagers *manager)
     m_manager = manager;
 }
 
+void PickBoundingVolumeJob::markPickersDirty()
+{
+    m_pickersDirty = true;
+}
+
 void PickBoundingVolumeJob::run()
 {
     Q_ASSERT(m_frameGraphRoot && m_renderSettings && m_node && m_manager);
@@ -231,49 +309,55 @@ void PickBoundingVolumeJob::dispatchPickEvents(const QMouseEvent &event,
                                                const PickingUtils::CollisionVisitor::HitList &sphereHits,
                                                QPickEvent::Buttons eventButton,
                                                int eventButtons,
-                                               int eventModifiers)
+                                               int eventModifiers,
+                                               bool trianglePickingRequested,
+                                               bool allHitsRequested)
 {
     ObjectPicker *lastCurrentPicker = m_manager->objectPickerManager()->data(m_currentPicker);
     // If we have hits
     if (!sphereHits.isEmpty()) {
-
-        const bool trianglePickingRequested = (m_renderSettings->pickResultMode() == QPickingSettings::AllPicks);
         // Note: how can we control that we want the first/last/all elements along the ray to be picked
 
         // How do we differentiate betwnee an Entity with no GeometryRenderer and one with one, both having
         // an ObjectPicker component when it comes
 
-        // We want to gather hits against triangles
-        // build a triangle based bounding volume
-
         for (const QCollisionQueryResult::Hit &hit : qAsConst(sphereHits)) {
             Entity *entity = m_manager->renderNodesManager()->lookupResource(hit.m_entityId);
-            HObjectPicker objectPickerHandle = entity->componentHandle<ObjectPicker, 16>();
+            HObjectPicker objectPickerHandle = entity->componentHandle<ObjectPicker>();
 
             // If the Entity which actually received the hit doesn't have
             // an object picker component, we need to check the parent if it has one ...
             while (objectPickerHandle.isNull() && entity != nullptr) {
                 entity = entity->parent();
                 if (entity != nullptr)
-                    objectPickerHandle = entity->componentHandle<ObjectPicker, 16>();
+                    objectPickerHandle = entity->componentHandle<ObjectPicker>();
             }
-            ObjectPicker *objectPicker = m_manager->objectPickerManager()->data(objectPickerHandle);
 
-            if (objectPicker != nullptr) {
+            ObjectPicker *objectPicker = m_manager->objectPickerManager()->data(objectPickerHandle);
+            if (objectPicker != nullptr && objectPicker->isEnabled()) {
+
+                if (lastCurrentPicker && !allHitsRequested) {
+                    // if there is a current picker, it will "grab" all events until released.
+                    // Clients should test that entity is what they expect (or only use
+                    // world coordinates)
+                    objectPicker = lastCurrentPicker;
+                }
+
                 // Send the corresponding event
                 QVector3D localIntersection = hit.m_intersection;
                 if (entity && entity->worldTransform())
-                    localIntersection = hit.m_intersection * entity->worldTransform()->inverted();
+                    localIntersection = entity->worldTransform()->inverted() * hit.m_intersection;
 
                 QPickEventPtr pickEvent;
-                if (trianglePickingRequested)
-                    pickEvent.reset(new QPickTriangleEvent(event.localPos(), hit.m_intersection, localIntersection, hit.m_distance,
+                if (trianglePickingRequested) {
+                    pickEvent = QSharedPointer<QPickTriangleEvent>::create(event.localPos(), hit.m_intersection, localIntersection, hit.m_distance,
                                                            hit.m_triangleIndex, hit.m_vertexIndex[0], hit.m_vertexIndex[1], hit.m_vertexIndex[2],
-                            eventButton, eventButtons, eventModifiers));
-                else
-                    pickEvent.reset(new QPickEvent(event.localPos(), hit.m_intersection, localIntersection, hit.m_distance,
-                                                   eventButton, eventButtons, eventModifiers));
-
+                            eventButton, eventButtons, eventModifiers, hit.m_uvw);
+                    QPickEventPrivate::get(pickEvent.data())->m_entity = hit.m_entityId;
+                } else {
+                    pickEvent = QSharedPointer<QPickEvent>::create(event.localPos(), hit.m_intersection, localIntersection, hit.m_distance,
+                                                   eventButton, eventButtons, eventModifiers);
+                }
                 switch (event.type()) {
                 case QEvent::MouseButtonPress: {
                     // Store pressed object handle
@@ -284,26 +368,26 @@ void PickBoundingVolumeJob::dispatchPickEvents(const QMouseEvent &event,
                 }
 
                 case QEvent::MouseButtonRelease: {
-                    if (lastCurrentPicker != nullptr && m_currentPicker == objectPickerHandle)
-                        m_currentPicker = HObjectPicker();
                     // Only send the release event if it was pressed
-                    if (objectPicker->isPressed()) {
-                        objectPicker->onClicked(pickEvent);
+                    if (objectPicker->isPressed())
                         objectPicker->onReleased(pickEvent);
+                    if (lastCurrentPicker != nullptr && m_currentPicker == objectPickerHandle) {
+                        objectPicker->onClicked(pickEvent);
+                        m_currentPicker = HObjectPicker();
                     }
                     break;
                 }
-
+#if QT_CONFIG(gestures)
                 case Qt::TapGesture: {
                     objectPicker->onClicked(pickEvent);
                     break;
                 }
-
+#endif
                 case QEvent::MouseMove: {
-                    if (objectPicker->isPressed() && objectPicker->isDragEnabled()) {
+                    if ((objectPicker->isPressed() || objectPicker->isHoverEnabled()) && objectPicker->isDragEnabled()) {
                         objectPicker->onMoved(pickEvent);
                     }
-                    // fallthrough
+                    Q_FALLTHROUGH(); // fallthrough
                 }
                 case QEvent::HoverMove: {
                     if (!m_hoveredPickers.contains(objectPickerHandle)) {
@@ -355,7 +439,7 @@ void PickBoundingVolumeJob::viewMatrixForCamera(Qt3DCore::QNodeId cameraId,
     if (camNode != nullptr &&
             (lens = camNode->renderComponent<CameraLens>()) != nullptr &&
             lens->isEnabled()) {
-        viewMatrix = *camNode->worldTransform();
+        viewMatrix = lens->viewMatrix(*camNode->worldTransform());
         projectionMatrix = lens->projection();
     }
 }
@@ -373,19 +457,39 @@ QRect PickBoundingVolumeJob::windowViewport(const QSize &area, const QRectF &rel
     return relativeViewport.toRect();
 }
 
-QRay3D PickBoundingVolumeJob::rayForViewportAndCamera(const QSize &area,
-                                                      const QPoint &pos,
-                                                      const QRectF &relativeViewport,
-                                                      Qt3DCore::QNodeId cameraId) const
+RayCasting::QRay3D PickBoundingVolumeJob::rayForViewportAndCamera(const PickingUtils::ViewportCameraAreaDetails &vca,
+                                                                  QObject *eventSource,
+                                                                  const QMouseEvent &event) const
 {
+    static RayCasting::QRay3D invalidRay({}, {}, 0.f);
+
     QMatrix4x4 viewMatrix;
     QMatrix4x4 projectionMatrix;
-    viewMatrixForCamera(cameraId, viewMatrix, projectionMatrix);
-    const QRect viewport = windowViewport(area, relativeViewport);
+    viewMatrixForCamera(vca.cameraId, viewMatrix, projectionMatrix);
+    const QRect viewport = windowViewport(vca.area, vca.viewport);
+    const QPoint pos = event.pos();
+
+    if (vca.area.isValid() && !viewport.contains(pos))
+        return invalidRay;
+    if (vca.surface) {
+        QSurface *surface = nullptr;
+        if (eventSource) {
+            QWindow *window = qobject_cast<QWindow *>(eventSource);
+            if (window) {
+                surface = static_cast<QSurface *>(window);
+            } else {
+                QOffscreenSurface *offscreen = qobject_cast<QOffscreenSurface *>(eventSource);
+                if (offscreen)
+                    surface = static_cast<QSurface *>(offscreen);
+            }
+        }
+        if (surface && vca.surface != surface)
+            return invalidRay;
+    }
 
     // In GL the y is inverted compared to Qt
-    const QPoint glCorrectPos = QPoint(pos.x(), area.isValid() ? area.height() - pos.y() : pos.y());
-    const QRay3D ray = intersectionRay(glCorrectPos, viewMatrix, projectionMatrix, viewport);
+    const QPoint glCorrectPos = QPoint(pos.x(), vca.area.isValid() ? vca.area.height() - pos.y() : pos.y());
+    const auto ray = intersectionRay(glCorrectPos, viewMatrix, projectionMatrix, viewport);
     return ray;
 }
 
