@@ -62,7 +62,11 @@
 #include <Qt3DRender/private/statesetnode_p.h>
 #include <Qt3DRender/private/dispatchcompute_p.h>
 #include <Qt3DRender/private/rendersurfaceselector_p.h>
+#include <Qt3DRender/private/rendercapture_p.h>
+#include <Qt3DRender/private/buffercapture_p.h>
 #include <Qt3DRender/private/stringtoint_p.h>
+#include <Qt3DRender/private/techniquemanager_p.h>
+#include <Qt3DRender/private/memorybarrier_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -161,6 +165,7 @@ void setRenderViewConfigFromFrameGraphLeafNode(RenderView *rv, const FrameGraphN
                 // a subregion relative to that of the parent viewport
                 const ViewportNode *vpNode = static_cast<const ViewportNode *>(node);
                 rv->setViewport(computeViewport(rv->viewport(), vpNode));
+                rv->setGamma(vpNode->gamma());
                 break;
             }
 
@@ -223,6 +228,29 @@ void setRenderViewConfigFromFrameGraphLeafNode(RenderView *rv, const FrameGraphN
                 }
                 break;
             }
+            case FrameGraphNode::RenderCapture: {
+                auto *renderCapture = const_cast<Render::RenderCapture *>(
+                                            static_cast<const Render::RenderCapture *>(node));
+                if (rv->renderCaptureNodeId().isNull() && renderCapture->wasCaptureRequested()) {
+                    renderCapture->acknowledgeCaptureRequest();
+                    rv->setRenderCaptureNodeId(renderCapture->peerId());
+                }
+                break;
+            }
+
+            case FrameGraphNode::MemoryBarrier: {
+                const Render::MemoryBarrier *barrier = static_cast<const Render::MemoryBarrier *>(node);
+                rv->setMemoryBarrier(barrier->waitOperations()|rv->memoryBarrier());
+                break;
+            }
+
+            case FrameGraphNode::BufferCapture: {
+                auto *bufferCapture = const_cast<Render::BufferCapture *>(
+                                            static_cast<const Render::BufferCapture *>(node));
+                if (bufferCapture != nullptr)
+                     rv->setIsDownloadBuffersEnable(bufferCapture->isEnabled());
+                break;
+            }
 
             default:
                 // Should never get here
@@ -247,67 +275,30 @@ Technique *findTechniqueForEffect(Renderer *renderer,
 
     NodeManagers *manager = renderer->nodeManagers();
     QVector<Technique*> matchingTechniques;
+    const bool hasInvalidTechniqueFilter = (techniqueFilter == nullptr || techniqueFilter->filters().isEmpty());
 
     // Iterate through the techniques in the effect
     const auto techniqueIds = effect->techniques();
     for (const QNodeId techniqueId : techniqueIds) {
         Technique *technique = manager->techniqueManager()->lookupResource(techniqueId);
 
-        if (!technique)
-            continue;
+        // Should be valid, if not there likely a problem with node addition/destruction changes
+        Q_ASSERT(technique);
 
-        // We need to be sure the renderer is still running <=> still has a GraphicsContext
-        if (renderer->isRunning() && *renderer->contextInfo() == *technique->graphicsApiFilter()) {
-
-            // If no techniqueFilter is present, we return the technique as it satisfies OpenGL version
-            bool foundMatch = (techniqueFilter == nullptr || techniqueFilter->filters().isEmpty());
-            if (foundMatch) {
-                matchingTechniques.append(technique);
-                continue;
-            }
-
-            // There is a technique filter so we need to check for a technique with suitable criteria.
-            // Check for early bail out if the technique doesn't have sufficient number of criteria and
-            // can therefore never satisfy the filter
-            if (technique->filterKeys().size() < techniqueFilter->filters().size())
-                continue;
-
-            // Iterate through the filter criteria and for each one search for a criteria on the
-            // technique that satisfies it
-            const auto filterKeyIds = techniqueFilter->filters();
-            for (const QNodeId filterKeyId : filterKeyIds) {
-                foundMatch = false;
-                FilterKey *filterKey = manager->filterKeyManager()->lookupResource(filterKeyId);
-
-                const auto techniqueFilterKeyIds = technique->filterKeys();
-                for (const QNodeId techniqueFilterKeyId : techniqueFilterKeyIds) {
-                    FilterKey *techniqueFilterKey = manager->filterKeyManager()->lookupResource(techniqueFilterKeyId);
-                    if ((foundMatch = (*techniqueFilterKey == *filterKey)))
-                        break;
-                }
-
-                if (!foundMatch) {
-                    // No match for TechniqueFilter criterion in any of the technique's criteria.
-                    // So no way this can match. Don't bother checking the rest of the criteria.
-                    break;
-                }
-            }
-
-            if (foundMatch) {
-                // All criteria matched - we have a winner!
-                matchingTechniques.append(technique);
-            }
-        }
+        // Check if the technique is compatible with the rendering API
+        // If no techniqueFilter is present, we return the technique as it satisfies OpenGL version
+        if (technique->isCompatibleWithRenderer() && (hasInvalidTechniqueFilter || technique->isCompatibleWithFilters(techniqueFilter->filters())))
+            matchingTechniques.append(technique);
     }
 
     if (matchingTechniques.size() == 0) // We failed to find a suitable technique to use :(
         return nullptr;
 
     if (matchingTechniques.size() == 1)
-        return matchingTechniques[0];
+        return matchingTechniques.first();
 
     // Several compatible techniques, return technique with highest major and minor version
-    Technique* highest = matchingTechniques[0];
+    Technique* highest = matchingTechniques.first();
     GraphicsApiFilterData filter = *highest->graphicsApiFilter();
     for (auto it = matchingTechniques.cbegin() + 1; it < matchingTechniques.cend(); ++it) {
         if (filter < *(*it)->graphicsApiFilter()) {
@@ -384,7 +375,7 @@ void addParametersForIds(ParameterInfoList *params, ParameterManager *manager,
         Parameter *param = manager->lookupResource(paramId);
         ParameterInfoList::iterator it = std::lower_bound(params->begin(), params->end(), param->nameId());
         if (it == params->end() || it->nameId != param->nameId())
-            params->insert(it, ParameterInfo(param->nameId(), param->value()));
+            params->insert(it, ParameterInfo(param->nameId(), param->uniformValue()));
     }
 }
 
@@ -411,8 +402,11 @@ void addToRenderStateSet(RenderStateSet *stateSet,
                          const QVector<Qt3DCore::QNodeId> stateIds,
                          RenderStateManager *manager)
 {
-    for (const Qt3DCore::QNodeId &stateId : stateIds)
-        stateSet->addState(manager->lookupResource(stateId)->impl());
+    for (const Qt3DCore::QNodeId &stateId : stateIds) {
+        RenderStateNode *node = manager->lookupResource(stateId);
+        if (node->isEnabled())
+            stateSet->addState(node->impl());
+    }
 }
 
 namespace {
@@ -423,7 +417,9 @@ const int qNodeIdTypeId = qMetaTypeId<QNodeId>();
 }
 
 UniformBlockValueBuilder::UniformBlockValueBuilder()
-    : shaderDataManager(nullptr)
+    : updatedPropertiesOnly(false)
+    , shaderDataManager(nullptr)
+    , textureManager(nullptr)
 {
 }
 
@@ -439,12 +435,16 @@ void UniformBlockValueBuilder::buildActiveUniformNameValueMapHelper(ShaderData *
         QVariantList list = value.value<QVariantList>();
         if (list.at(0).userType() == qNodeIdTypeId) { // Array of struct qmlPropertyName[i].structMember
             for (int i = 0; i < list.size(); ++i) {
+                const QVariant variantElement = list.at(i);
                 if (list.at(i).userType() == qNodeIdTypeId) {
-                    ShaderData *subShaderData = shaderDataManager->lookupResource(list.at(i).value<QNodeId>());
-                    if (subShaderData)
+                    const auto nodeId = variantElement.value<QNodeId>();
+                    ShaderData *subShaderData = shaderDataManager->lookupResource(nodeId);
+                    if (subShaderData) {
                         buildActiveUniformNameValueMapStructHelper(subShaderData,
                                                                    blockName + QLatin1Char('.') + qmlPropertyName + blockArray.arg(i),
                                                                    QLatin1String(""));
+                    }
+                    // Note: we only handle ShaderData as nested container nodes here
                 }
             }
         } else { // Array of scalar/vec  qmlPropertyName[0]
@@ -455,11 +455,16 @@ void UniformBlockValueBuilder::buildActiveUniformNameValueMapHelper(ShaderData *
             }
         }
     } else if (value.userType() == qNodeIdTypeId) { // Struct qmlPropertyName.structMember
-        ShaderData *rSubShaderData = shaderDataManager->lookupResource(value.value<QNodeId>());
-        if (rSubShaderData)
+        const auto nodeId = value.value<QNodeId>();
+        ShaderData *rSubShaderData = shaderDataManager->lookupResource(nodeId);
+        if (rSubShaderData) {
             buildActiveUniformNameValueMapStructHelper(rSubShaderData,
                                                        blockName,
                                                        qmlPropertyName);
+        } else if (textureManager->contains(nodeId)) {
+            const auto varId = StringToInt::lookupId(blockName + QLatin1Char('.') + qmlPropertyName);
+            activeUniformNamesToValue.insert(varId, value);
+        }
     } else { // Scalar / Vec
         QString varName = blockName + QLatin1Char('.') + qmlPropertyName;
         if (uniforms.contains(varName)) {
@@ -493,7 +498,7 @@ void UniformBlockValueBuilder::buildActiveUniformNameValueMapStructHelper(Shader
     }
 }
 
-ParameterInfo::ParameterInfo(const int nameId, const QVariant &value)
+ParameterInfo::ParameterInfo(const int nameId, const UniformValue &value)
     : nameId(nameId)
     , value(value)
 {}
