@@ -45,17 +45,17 @@ namespace Qt3DRender {
 
 namespace Render {
 
-Q_LOGGING_CATEGORY(Backend, "Qt3D.Renderer.Backend")
-Q_LOGGING_CATEGORY(Frontend, "Qt3D.Renderer.Frontend")
-Q_LOGGING_CATEGORY(Io, "Qt3D.Renderer.IO")
-Q_LOGGING_CATEGORY(Jobs, "Qt3D.Renderer.Jobs")
-Q_LOGGING_CATEGORY(Framegraph, "Qt3D.Renderer.Framegraph")
-Q_LOGGING_CATEGORY(RenderNodes, "Qt3D.Renderer.RenderNodes")
-Q_LOGGING_CATEGORY(Rendering, "Qt3D.Renderer.Rendering")
-Q_LOGGING_CATEGORY(Memory, "Qt3D.Renderer.Memory")
-Q_LOGGING_CATEGORY(Shaders, "Qt3D.Renderer.Shaders")
-Q_LOGGING_CATEGORY(RenderStates, "Qt3D.Renderer.RenderStates")
-Q_LOGGING_CATEGORY(VSyncAdvanceService, "Qt3D.Renderer.VsyncAdvanceService")
+Q_LOGGING_CATEGORY(Backend, "Qt3D.Renderer.Backend", QtWarningMsg)
+Q_LOGGING_CATEGORY(Frontend, "Qt3D.Renderer.Frontend", QtWarningMsg)
+Q_LOGGING_CATEGORY(Io, "Qt3D.Renderer.IO", QtWarningMsg)
+Q_LOGGING_CATEGORY(Jobs, "Qt3D.Renderer.Jobs", QtWarningMsg)
+Q_LOGGING_CATEGORY(Framegraph, "Qt3D.Renderer.Framegraph", QtWarningMsg)
+Q_LOGGING_CATEGORY(RenderNodes, "Qt3D.Renderer.RenderNodes", QtWarningMsg)
+Q_LOGGING_CATEGORY(Rendering, "Qt3D.Renderer.Rendering", QtWarningMsg)
+Q_LOGGING_CATEGORY(Memory, "Qt3D.Renderer.Memory", QtWarningMsg)
+Q_LOGGING_CATEGORY(Shaders, "Qt3D.Renderer.Shaders", QtWarningMsg)
+Q_LOGGING_CATEGORY(RenderStates, "Qt3D.Renderer.RenderStates", QtWarningMsg)
+Q_LOGGING_CATEGORY(VSyncAdvanceService, "Qt3D.Renderer.VsyncAdvanceService", QtWarningMsg)
 
 } // namespace Render
 
