@@ -97,15 +97,14 @@ void GLTexture::destroyGLTexture()
 {
     delete m_gl;
     m_gl = nullptr;
-    QMutexLocker locker(&m_dirtyFlagMutex);
-    m_dirty = 0;
+    m_dirtyFlags.store(0);
 
     destroyResources();
 }
 
 QOpenGLTexture* GLTexture::getOrCreateGLTexture()
 {
-    QMutexLocker locker(&m_dirtyFlagMutex);
+    QMutexLocker locker(&m_textureMutex);
     bool needUpload = false;
     bool texturedDataInvalid = false;
 
@@ -120,7 +119,7 @@ QOpenGLTexture* GLTexture::getOrCreateGLTexture()
             if (m_properties.target != QAbstractTexture::TargetAutomatic)
                 qWarning() << "[Qt3DRender::GLTexture] When a texture provides a generator, it's target is expected to be TargetAutomatic";
 
-            m_properties.target = m_textureData->target();
+            m_actualTarget = m_textureData->target();
             m_properties.width = m_textureData->width();
             m_properties.height = m_textureData->height();
             m_properties.depth = m_textureData->depth();
@@ -135,16 +134,16 @@ QOpenGLTexture* GLTexture::getOrCreateGLTexture()
                     m_properties.mipLevels = imageData.first()->mipLevels();
             }
 
-            m_dirty |= Properties;
+            setDirtyFlag(Properties, true);
             needUpload = true;
         } else {
-            qWarning() << "[Qt3DRender::GLTexture] No QTextureData generated from Texture Generator";
-            texturedDataInvalid = true;
+            qWarning() << "[Qt3DRender::GLTexture] No QTextureData generated from Texture Generator yet. Texture will be invalid for this frame";
+            return nullptr;
         }
     }
 
     // additional texture images may be defined through image data generators
-    if (m_dirty.testFlag(TextureData)) {
+    if (testDirtyFlag(TextureData)) {
         m_imageData.clear();
         needUpload = true;
 
@@ -168,24 +167,32 @@ QOpenGLTexture* GLTexture::getOrCreateGLTexture()
                         m_properties.format = static_cast<QAbstractTexture::TextureFormat>(imgData->format());
                     }
 
-                    m_dirty |= Properties;
+                    setDirtyFlag(Properties, true);
                 }
             } else {
-                qWarning() << "[Qt3DRender::GLTexture] No QTextureImageData generated from functor";
+                qWarning() << "[Qt3DRender::GLTexture] No QTextureImageData generated from functor yet, texture will be invalid for this frame";
                 texturedDataInvalid = true;
             }
         }
     }
 
+    // don't try to create the texture if the format was not set
+    if (m_properties.format == QAbstractTexture::Automatic)
+        return nullptr;
+
+    if (texturedDataInvalid)
+        return nullptr;
 
     // if the properties changed, we need to re-allocate the texture
-    if (m_dirty.testFlag(Properties)) {
+    if (testDirtyFlag(Properties)) {
         delete m_gl;
         m_gl = nullptr;
     }
 
     if (!m_gl) {
         m_gl = buildGLTexture();
+        if (!m_gl)
+            return nullptr;
         m_gl->allocateStorage();
         if (!m_gl->isStorageAllocated()) {
             qWarning() << Q_FUNC_INFO << "texture storage allocation failed";
@@ -196,33 +203,38 @@ QOpenGLTexture* GLTexture::getOrCreateGLTexture()
     // need to (re-)upload texture data?
     if (needUpload && !texturedDataInvalid) {
         uploadGLTextureData();
+        setDirtyFlag(TextureData, false);
     }
 
     // need to set texture parameters?
-    if (m_dirty.testFlag(Properties) || m_dirty.testFlag(Parameters)) {
+    if (testDirtyFlag(Properties) || testDirtyFlag(Parameters)) {
         updateGLTextureParameters();
     }
 
-    m_dirty = 0;
+    // un-set properties and parameters. The TextureData flag might have been set by another thread
+    // in the meantime, so don't clear that.
+    setDirtyFlag(Properties, false);
+    setDirtyFlag(Parameters, false);
 
     return m_gl;
 }
 
 void GLTexture::setParameters(const TextureParameters &params)
 {
+    QMutexLocker locker(&m_textureMutex);
     if (m_parameters != params) {
         m_parameters = params;
-        QMutexLocker locker(&m_dirtyFlagMutex);
-        m_dirty |= Parameters;
+        setDirtyFlag(Parameters);
     }
 }
 
 void GLTexture::setProperties(const TextureProperties &props)
 {
+    QMutexLocker locker(&m_textureMutex);
     if (m_properties != props) {
         m_properties = props;
-        QMutexLocker locker(&m_dirtyFlagMutex);
-        m_dirty |= Properties;
+        m_actualTarget = props.target;
+        setDirtyFlag(Properties);
     }
 }
 
@@ -276,6 +288,22 @@ void GLTexture::setImages(const QVector<Image> &images)
     }
 }
 
+void GLTexture::setGenerator(const QTextureGeneratorPtr &generator)
+{
+    if (m_dataFunctor != generator) {
+        if (m_dataFunctor)
+            m_textureDataManager->releaseData(m_dataFunctor, this);
+
+        m_textureData.reset();
+        m_dataFunctor = generator;
+
+        if (m_dataFunctor) {
+            m_textureDataManager->requestData(m_dataFunctor, this);
+            requestUpload();
+        }
+    }
+}
+
 QOpenGLTexture *GLTexture::buildGLTexture()
 {
     QOpenGLContext *ctx = QOpenGLContext::currentContext();
@@ -284,12 +312,12 @@ QOpenGLTexture *GLTexture::buildGLTexture()
         return nullptr;
     }
 
-    if (m_properties.target == QAbstractTexture::TargetAutomatic) {
+    if (m_actualTarget == QAbstractTexture::TargetAutomatic) {
         qWarning() << Q_FUNC_INFO << "something went wrong, target shouldn't be automatic at this point";
         return nullptr;
     }
 
-    QOpenGLTexture* glTex = new QOpenGLTexture(static_cast<QOpenGLTexture::Target>(m_properties.target));
+    QOpenGLTexture* glTex = new QOpenGLTexture(static_cast<QOpenGLTexture::Target>(m_actualTarget));
 
     // m_format may not be ES2 compatible. Now it's time to convert it, if necessary.
     QAbstractTexture::TextureFormat format = m_properties.format;
@@ -327,16 +355,16 @@ QOpenGLTexture *GLTexture::buildGLTexture()
                          static_cast<QOpenGLTexture::TextureFormat>(format));
     glTex->setSize(m_properties.width, m_properties.height, m_properties.depth);
     // Set layers count if texture array
-    if (m_properties.target == QAbstractTexture::Target1DArray ||
-            m_properties.target == QAbstractTexture::Target2DArray ||
-            m_properties.target == QAbstractTexture::Target3D ||
-            m_properties.target == QAbstractTexture::Target2DMultisampleArray ||
-            m_properties.target == QAbstractTexture::TargetCubeMapArray) {
+    if (m_actualTarget == QAbstractTexture::Target1DArray ||
+            m_actualTarget == QAbstractTexture::Target2DArray ||
+            m_actualTarget == QAbstractTexture::Target3D ||
+            m_actualTarget == QAbstractTexture::Target2DMultisampleArray ||
+            m_actualTarget == QAbstractTexture::TargetCubeMapArray) {
         glTex->setLayers(m_properties.layers);
     }
 
-    if (m_properties.target == QAbstractTexture::Target2DMultisample ||
-            m_properties.target == QAbstractTexture::Target2DMultisampleArray) {
+    if (m_actualTarget == QAbstractTexture::Target2DMultisample ||
+            m_actualTarget == QAbstractTexture::Target2DMultisampleArray) {
         // Set samples count if multisampled texture
         // (multisampled textures don't have mipmaps)
         glTex->setSamples(m_properties.samples);
@@ -382,7 +410,7 @@ void GLTexture::uploadGLTextureData()
             for (int layer = 0; layer < data->layers(); layer++) {
                 for (int face = 0; face < data->faces(); face++) {
                     for (int level = 0; level < mipLevels; level++) {
-                        // ensure we don't accidently cause a detach / copy of the raw bytes
+                        // ensure we don't accidentally cause a detach / copy of the raw bytes
                         const QByteArray bytes(data->data(layer, face, level));
                         uploadGLData(m_gl, level, layer,
                                      static_cast<QOpenGLTexture::CubeMapFace>(QOpenGLTexture::CubeMapPositiveX + face),
@@ -397,7 +425,7 @@ void GLTexture::uploadGLTextureData()
     for (int i = 0; i < m_images.size(); i++) {
         const QTextureImageDataPtr &imgData = m_imageData.at(i);
 
-        // ensure we don't accidently cause a detach / copy of the raw bytes
+        // ensure we don't accidentally cause a detach / copy of the raw bytes
         const QByteArray bytes(imgData->data());
         uploadGLData(m_gl, m_images[i].mipLevel, m_images[i].layer,
                      static_cast<QOpenGLTexture::CubeMapFace>(m_images[i].face),
@@ -408,11 +436,11 @@ void GLTexture::uploadGLTextureData()
 void GLTexture::updateGLTextureParameters()
 {
     m_gl->setWrapMode(QOpenGLTexture::DirectionS, static_cast<QOpenGLTexture::WrapMode>(m_parameters.wrapModeX));
-    if (m_properties.target != QAbstractTexture::Target1D &&
-            m_properties.target != QAbstractTexture::Target1DArray &&
-            m_properties.target != QAbstractTexture::TargetBuffer)
+    if (m_actualTarget != QAbstractTexture::Target1D &&
+            m_actualTarget != QAbstractTexture::Target1DArray &&
+            m_actualTarget != QAbstractTexture::TargetBuffer)
         m_gl->setWrapMode(QOpenGLTexture::DirectionT, static_cast<QOpenGLTexture::WrapMode>(m_parameters.wrapModeY));
-    if (m_properties.target == QAbstractTexture::Target3D)
+    if (m_actualTarget == QAbstractTexture::Target3D)
         m_gl->setWrapMode(QOpenGLTexture::DirectionR, static_cast<QOpenGLTexture::WrapMode>(m_parameters.wrapModeZ));
     m_gl->setMinMagFilters(static_cast<QOpenGLTexture::Filter>(m_parameters.minificationFilter),
                            static_cast<QOpenGLTexture::Filter>(m_parameters.magnificationFilter));
