@@ -29,10 +29,11 @@
 #include <QtTest/QTest>
 #include <Qt3DRender/qrendertargetoutput.h>
 #include <Qt3DRender/private/uniform_p.h>
-#include <Qt3DRender/private/graphicshelpergl3_3_p.h>
+#include <Qt3DRender/private/graphicshelpergl3_2_p.h>
 #include <Qt3DRender/private/attachmentpack_p.h>
-#include <QOpenGLFunctions_3_3_Core>
+#include <QOpenGLFunctions_3_2_Core>
 #include <QOpenGLShaderProgram>
+#include <QSurfaceFormat>
 
 #if !defined(QT_OPENGL_ES_2) && defined(QT_OPENGL_3_2)
 
@@ -44,7 +45,7 @@ using namespace Qt3DRender::Render;
 namespace {
 
 const QByteArray vertCode = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "in vec3 vertexPosition;\n" \
             "in vec2 vertexTexCoord;\n" \
             "out vec2 texCoord;\n" \
@@ -55,7 +56,7 @@ const QByteArray vertCode = QByteArrayLiteral(
             "}\n");
 
 const QByteArray vertCodeUniformBuffer = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "in vec3 vertexPosition;\n" \
             "in vec2 vertexTexCoord;\n" \
             "in int vertexColorIndex;\n" \
@@ -69,7 +70,7 @@ const QByteArray vertCodeUniformBuffer = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeFragOutputs = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "out vec2 temp;\n" \
             "void main()\n" \
@@ -79,7 +80,7 @@ const QByteArray fragCodeFragOutputs = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsFloat = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "uniform float multiplier;\n" \
             "uniform vec2 multiplierVec2;\n" \
@@ -92,8 +93,8 @@ const QByteArray fragCodeUniformsFloat = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsInt = QByteArrayLiteral(
-            "#version 330 core\n" \
-            "out ivec4 color;\n" \
+            "#version 150 core\n" \
+            "out vec4 color;\n" \
             "uniform int multiplier;\n" \
             "uniform ivec2 multiplierVec2;\n" \
             "uniform ivec3 multiplierVec3;\n" \
@@ -105,8 +106,8 @@ const QByteArray fragCodeUniformsInt = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsUInt = QByteArrayLiteral(
-            "#version 330 core\n" \
-            "out uvec4 color;\n" \
+            "#version 150 core\n" \
+            "out vec4 color;\n" \
             "uniform uint multiplier;\n" \
             "uniform uvec2 multiplierVec2;\n" \
             "uniform uvec3 multiplierVec3;\n" \
@@ -118,7 +119,7 @@ const QByteArray fragCodeUniformsUInt = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformsFloatMatrices = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "uniform mat2 m2;\n" \
             "uniform mat2x3 m23;\n" \
@@ -136,7 +137,7 @@ const QByteArray fragCodeUniformsFloatMatrices = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeUniformBuffer = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "out vec4 color;\n" \
             "flat in int colorIndex;\n" \
             "uniform ColorArray\n" \
@@ -149,7 +150,7 @@ const QByteArray fragCodeUniformBuffer = QByteArrayLiteral(
             "}\n");
 
 const QByteArray fragCodeSamplers = QByteArrayLiteral(
-            "#version 330 core\n" \
+            "#version 150 core\n" \
             "in vec2 texCoord;\n" \
             "out vec4 color;\n" \
             "uniform sampler1D s1;\n" \
@@ -171,7 +172,7 @@ const QByteArray fragCodeSamplers = QByteArrayLiteral(
 
 } // anonymous
 
-class tst_GraphicsHelperGL3_3 : public QObject
+class tst_GraphicsHelperGL3_2 : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
@@ -181,9 +182,10 @@ private Q_SLOTS:
         m_window.reset(new QWindow);
         m_window->setSurfaceType(QWindow::OpenGLSurface);
         m_window->setGeometry(0, 0, 10, 10);
+        m_window->create();
 
         QSurfaceFormat format;
-        format.setVersion(3, 3);
+        format.setVersion(3, 2);
         format.setProfile(QSurfaceFormat::CoreProfile);
         format.setDepthBufferSize(24);
         format.setSamples(4);
@@ -191,19 +193,17 @@ private Q_SLOTS:
         m_window->setFormat(format);
         m_glContext.setFormat(format);
 
-        m_window->create();
-
         if (!m_glContext.create()) {
             qWarning() << "Failed to create OpenGL context";
             return;
         }
 
         if (!m_glContext.makeCurrent(m_window.data())) {
-            qWarning() << "Failed to maed OpenGL context current";
+            qWarning() << "Failed to make OpenGL context current";
             return;
         }
 
-        if ((m_func = m_glContext.versionFunctions<QOpenGLFunctions_3_3_Core>()) != nullptr) {
+        if ((m_func = m_glContext.versionFunctions<QOpenGLFunctions_3_2_Core>()) != nullptr) {
             m_glHelper.initializeHelper(&m_glContext, m_func);
             m_initializationSuccessful = true;
         }
@@ -217,14 +217,14 @@ private Q_SLOTS:
     void alphaTest()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
         // Deprecated
     }
 
     void bindBufferBase()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint bufferId = 0;
@@ -252,7 +252,7 @@ private Q_SLOTS:
     void bindFragDataLocation()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -278,155 +278,57 @@ private Q_SLOTS:
     void bindFrameBufferAttachment()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
-        {
-            // GIVEN
-            GLuint fboId;
-            m_func->glGenFramebuffers(1, &fboId);
+        // GIVEN
+        GLuint fboId;
+        m_func->glGenFramebuffers(1, &fboId);
 
-            Attachment attachment;
-            attachment.m_point = QRenderTargetOutput::Color0;
+        Attachment attachment;
+        attachment.m_point = QRenderTargetOutput::Color0;
 
-            GLint maxAttachmentsCount = 0;
-            m_func->glGetIntegerv(GL_MAX_COLOR_ATTACHMENTS, &maxAttachmentsCount);
+        // THEN
+        QVERIFY(fboId != 0);
 
-            // THEN
-            QVERIFY(fboId != 0);
-            QVERIFY(maxAttachmentsCount >= 3);
+        // WHEN
+        m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
 
-            // WHEN
-            m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
+        QOpenGLTexture texture(QOpenGLTexture::Target2D);
+        texture.setSize(512, 512);
+        texture.setFormat(QOpenGLTexture::RGBA32F);
+        texture.setMinificationFilter(QOpenGLTexture::Linear);
+        texture.setMagnificationFilter(QOpenGLTexture::Linear);
+        texture.setWrapMode(QOpenGLTexture::ClampToEdge);
+        if (!texture.create())
+            qWarning() << "Texture creation failed";
+        texture.allocateStorage();
+        QVERIFY(texture.isStorageAllocated());
+        GLint error = m_func->glGetError();
+        QVERIFY(error == 0);
+        m_glHelper.bindFrameBufferAttachment(&texture, attachment);
 
-            QOpenGLTexture texture(QOpenGLTexture::Target2D);
-            texture.setSize(512, 512);
-            texture.setFormat(QOpenGLTexture::RGBA8U);
-            texture.setMinificationFilter(QOpenGLTexture::Linear);
-            texture.setMagnificationFilter(QOpenGLTexture::Linear);
-            texture.setWrapMode(QOpenGLTexture::ClampToEdge);
-            if (!texture.create())
-                qWarning() << "Texture creation failed";
-            texture.allocateStorage();
-            QVERIFY(texture.isStorageAllocated());
-            GLint error = m_func->glGetError();
-            QVERIFY(error == 0);
-            m_glHelper.bindFrameBufferAttachment(&texture, attachment);
+        // THEN
+        GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
+        QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
 
-            // THEN
-            GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
-            QCOMPARE(int(status), GL_FRAMEBUFFER_COMPLETE);
+        error = m_func->glGetError();
+        QVERIFY(error == 0);
+        GLint textureAttachmentId = 0;
+        m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
+                                                      GL_COLOR_ATTACHMENT0,
+                                                      GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME,
+                                                      &textureAttachmentId);
+        QCOMPARE(GLuint(textureAttachmentId), texture.textureId());
 
-            error = m_func->glGetError();
-            QVERIFY(error == 0);
-            GLint textureAttachmentId = 0;
-            m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                          GL_COLOR_ATTACHMENT0,
-                                                          GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME,
-                                                          &textureAttachmentId);
-            QCOMPARE(GLuint(textureAttachmentId), texture.textureId());
-
-            // Restore state
-            m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
-            m_func->glDeleteFramebuffers(1, &fboId);
-        }
-        {
-            // GIVEN
-            QOpenGLTexture texture(QOpenGLTexture::TargetCubeMap);
-            texture.setSize(512, 512);
-            texture.setFormat(QOpenGLTexture::RGBA32F);
-            texture.setMinificationFilter(QOpenGLTexture::Linear);
-            texture.setMagnificationFilter(QOpenGLTexture::Linear);
-            texture.setWrapMode(QOpenGLTexture::ClampToEdge);
-            if (!texture.create())
-                qWarning() << "Texture creation failed";
-            texture.allocateStorage();
-            QVERIFY(texture.isStorageAllocated());
-            GLint error = m_func->glGetError();
-            QVERIFY(error == 0);
-
-            { // Check All Faces
-
-                // GIVEN
-                GLuint fboId;
-                m_func->glGenFramebuffers(1, &fboId);
-
-                // THEN
-                QVERIFY(fboId != 0);
-
-                // WHEN
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
-
-                Attachment attachment;
-                attachment.m_point = QRenderTargetOutput::Color0;
-                attachment.m_face = Qt3DRender::QAbstractTexture::AllFaces;
-
-                m_glHelper.bindFrameBufferAttachment(&texture, attachment);
-
-                // THEN
-                GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
-                QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
-
-                error = m_func->glGetError();
-                QVERIFY(error == 0);
-                GLint textureIsLayered = 0;
-                m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                              GL_COLOR_ATTACHMENT0,
-                                                              GL_FRAMEBUFFER_ATTACHMENT_LAYERED,
-                                                              &textureIsLayered);
-                QCOMPARE(textureIsLayered, GL_TRUE);
-
-                // Restore state
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
-                m_func->glDeleteFramebuffers(1, &fboId);
-            }
-            { // Check Specific Faces
-
-                // GIVEN
-                GLuint fboId;
-                m_func->glGenFramebuffers(1, &fboId);
-
-                // THEN
-                QVERIFY(fboId != 0);
-
-                // WHEN
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fboId);
-
-                Attachment attachment;
-                attachment.m_point = QRenderTargetOutput::Color0;
-                attachment.m_face = Qt3DRender::QAbstractTexture::CubeMapNegativeZ;
-
-                m_glHelper.bindFrameBufferAttachment(&texture, attachment);
-
-                // THEN
-                GLenum status = m_func->glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER);
-                QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
-
-                error = m_func->glGetError();
-                QVERIFY(error == 0);
-                GLint textureFace = 0;
-                m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                              GL_COLOR_ATTACHMENT0,
-                                                              GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE,
-                                                              &textureFace);
-                QCOMPARE(textureFace, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z);
-                GLint textureIsLayered = 0;
-                m_func->glGetFramebufferAttachmentParameteriv(GL_DRAW_FRAMEBUFFER,
-                                                              GL_COLOR_ATTACHMENT0,
-                                                              GL_FRAMEBUFFER_ATTACHMENT_LAYERED,
-                                                              &textureIsLayered);
-                QCOMPARE(textureIsLayered, GL_FALSE);
-
-                // Restore state
-                m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
-                m_func->glDeleteFramebuffers(1, &fboId);
-            }
-        }
+        // Restore state
+        m_func->glBindFramebuffer(GL_DRAW_FRAMEBUFFER, 0);
+        m_func->glDeleteFramebuffers(1, &fboId);
     }
 
     void bindFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -436,15 +338,35 @@ private Q_SLOTS:
         QVERIFY(fboId != 0);
 
         // WHEN
-        m_glHelper.bindFrameBufferObject(fboId);
+        m_glHelper.bindFrameBufferObject(fboId, GraphicsHelperInterface::FBODraw);
 
         // THEN
-        const GLint error = m_func->glGetError();
+        GLint error = m_func->glGetError();
         QVERIFY(error == 0);
         GLint boundindFBOId = 0;
         m_func->glGetIntegerv(GL_DRAW_FRAMEBUFFER_BINDING, &boundindFBOId);
         QVERIFY(GLuint(boundindFBOId) == fboId);
 
+        // WHEN
+        m_glHelper.bindFrameBufferObject(fboId, GraphicsHelperInterface::FBORead);
+
+        // THEN
+        error = m_func->glGetError();
+        QVERIFY(error == 0);
+        boundindFBOId = 0;
+        m_func->glGetIntegerv(GL_READ_FRAMEBUFFER_BINDING, &boundindFBOId);
+        QVERIFY(GLuint(boundindFBOId) == fboId);
+
+        // WHEN
+        m_glHelper.bindFrameBufferObject(fboId, GraphicsHelperInterface::FBOReadAndDraw);
+
+        // THEN
+        error = m_func->glGetError();
+        QVERIFY(error == 0);
+        boundindFBOId = 0;
+        m_func->glGetIntegerv(GL_FRAMEBUFFER_BINDING, &boundindFBOId);
+        QVERIFY(GLuint(boundindFBOId) == fboId);
+
         // Cleanup
         m_func->glDeleteFramebuffers(1, &fboId);
     }
@@ -452,14 +374,14 @@ private Q_SLOTS:
     void bindShaderStorageBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
-        // Not supported in OpenGL 3.3
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
+        // Not supported in OpenGL 3.2
     }
 
     void bindUniformBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -479,7 +401,7 @@ private Q_SLOTS:
     void blendEquation()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLint equation = 0;
@@ -497,22 +419,22 @@ private Q_SLOTS:
     void blendFunci()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
-        // Not supported by OpenGL 3.3
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
+        // Not supported by OpenGL 3.2
     }
 
     void blendFuncSeparatei()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
-        // Not supported by OpenGL 3.3
+        // Not supported by OpenGL 3.2
     }
 
     void boundFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -537,7 +459,7 @@ private Q_SLOTS:
     void checkFrameBufferComplete()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
         // GIVEN
         GLuint fboId;
         m_func->glGenFramebuffers(1, &fboId);
@@ -570,7 +492,7 @@ private Q_SLOTS:
     void clearBufferf()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         // GIVEN
@@ -626,7 +548,6 @@ private Q_SLOTS:
         textures[3]->bind();
         m_func->glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_FLOAT, colors.data());
         textures[3]->release();
-
         for (const QVector4D c : colors) {
             QVERIFY(c == clearValue1);
         }
@@ -652,7 +573,7 @@ private Q_SLOTS:
     void createFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // WHEN
         const GLuint fboId = m_glHelper.createFrameBufferObject();
@@ -667,7 +588,7 @@ private Q_SLOTS:
     void depthMask()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLboolean depthWritingEnabled = false;
@@ -694,7 +615,7 @@ private Q_SLOTS:
     void depthTest()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_DEPTH_TEST);
@@ -719,7 +640,7 @@ private Q_SLOTS:
     void disableClipPlane()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnable(GL_CLIP_DISTANCE0 + 5);
@@ -737,7 +658,7 @@ private Q_SLOTS:
     void disablei()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnablei(GL_BLEND, 2);
@@ -755,7 +676,7 @@ private Q_SLOTS:
     void disablePrimitiveRestart()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnable(GL_PRIMITIVE_RESTART);
@@ -773,7 +694,7 @@ private Q_SLOTS:
     void drawBuffers()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -845,7 +766,7 @@ private Q_SLOTS:
     void enableClipPlane()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_CLIP_DISTANCE0 + 4);
@@ -863,7 +784,7 @@ private Q_SLOTS:
     void enablei()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisablei(GL_BLEND, 4);
@@ -881,7 +802,7 @@ private Q_SLOTS:
     void enablePrimitiveRestart()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_PRIMITIVE_RESTART);
@@ -905,7 +826,7 @@ private Q_SLOTS:
     void frontFace()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glFrontFace(GL_CW);
@@ -926,7 +847,7 @@ private Q_SLOTS:
     void getRenderBufferDimensions()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint renderBufferId = 0;
@@ -949,7 +870,7 @@ private Q_SLOTS:
     void getTextureDimensions()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLTexture texture(QOpenGLTexture::Target2D);
@@ -970,7 +891,7 @@ private Q_SLOTS:
     void pointSize()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glEnable(GL_PROGRAM_POINT_SIZE);
@@ -993,7 +914,7 @@ private Q_SLOTS:
     void maxClipPlaneCount()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLint maxCount = -1;
@@ -1006,7 +927,7 @@ private Q_SLOTS:
     void programUniformBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1034,7 +955,7 @@ private Q_SLOTS:
     void programAttributesAndLocations()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1065,7 +986,7 @@ private Q_SLOTS:
     void programUniformsAndLocations()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1124,15 +1045,15 @@ private Q_SLOTS:
     void programShaderStorageBlock()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
-        // Not supported in 3.3
+        // Not supported in 3.2
     }
 
     void releaseFrameBufferObject()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         GLuint fboId;
@@ -1151,7 +1072,7 @@ private Q_SLOTS:
     void setMSAAEnabled()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_MULTISAMPLE);
@@ -1175,7 +1096,7 @@ private Q_SLOTS:
     void setAlphaCoverageEnabled()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);
@@ -1199,7 +1120,7 @@ private Q_SLOTS:
     void setClipPlane()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // Deprecated in 3.3 core
     }
@@ -1207,7 +1128,7 @@ private Q_SLOTS:
     void setSeamlessCubemap()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
@@ -1229,7 +1150,7 @@ private Q_SLOTS:
     void setVerticesPerPatch()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         m_func->glDisable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
@@ -1317,7 +1238,7 @@ private Q_SLOTS:
     void uniformsByteSize()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QFETCH(QByteArray, fragShader);
@@ -1358,11 +1279,10 @@ private Q_SLOTS:
         m_func->glUseProgram(0);
     }
 
-
     void useProgram()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1394,13 +1314,14 @@ private Q_SLOTS:
     void vertexAttribDivisor()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
+        // Not available in 3.2
     }
 
     void glUniform1fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1426,7 +1347,7 @@ private Q_SLOTS:
     void glUniform2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1453,7 +1374,7 @@ private Q_SLOTS:
     void glUniform3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1480,7 +1401,7 @@ private Q_SLOTS:
     void glUniform4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1507,7 +1428,7 @@ private Q_SLOTS:
     void glUniform1iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1533,7 +1454,7 @@ private Q_SLOTS:
     void glUniform2iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1560,7 +1481,7 @@ private Q_SLOTS:
     void glUniform3iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1587,7 +1508,7 @@ private Q_SLOTS:
     void glUniform4iv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1614,7 +1535,7 @@ private Q_SLOTS:
     void glUniform1uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1640,7 +1561,7 @@ private Q_SLOTS:
     void glUniform2uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1667,7 +1588,7 @@ private Q_SLOTS:
     void glUniform3uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1694,7 +1615,7 @@ private Q_SLOTS:
     void glUniform4uiv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1721,7 +1642,7 @@ private Q_SLOTS:
     void glUniformMatrix2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1748,7 +1669,7 @@ private Q_SLOTS:
     void glUniformMatrix3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1775,7 +1696,7 @@ private Q_SLOTS:
     void glUniformMatrix4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1802,7 +1723,7 @@ private Q_SLOTS:
     void glUniformMatrix2x3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1829,7 +1750,7 @@ private Q_SLOTS:
     void glUniformMatrix3x2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1856,7 +1777,7 @@ private Q_SLOTS:
     void glUniformMatrix2x4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1883,7 +1804,7 @@ private Q_SLOTS:
     void glUniformMatrix4x2fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1910,7 +1831,7 @@ private Q_SLOTS:
     void glUniformMatrix3x4fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1937,7 +1858,7 @@ private Q_SLOTS:
     void glUniformMatrix4x3fv()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         // GIVEN
         QOpenGLShaderProgram shaderProgram;
@@ -1961,11 +1882,10 @@ private Q_SLOTS:
         m_func->glUseProgram(0);
     }
 
-
     void blitFramebuffer()
     {
         if (!m_initializationSuccessful)
-            QSKIP("Initialization failed, OpenGL 3.3 Core functions not supported");
+            QSKIP("Initialization failed, OpenGL 3.2 Core functions not supported");
 
         GLint maxSamples;
         m_func->glGetIntegerv(GL_MAX_SAMPLES, &maxSamples);
@@ -1991,13 +1911,13 @@ private Q_SLOTS:
         m_func->glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, fboTextures[1], 0);
 
         GLenum status = m_func->glCheckFramebufferStatus(GL_FRAMEBUFFER);
-        QCOMPARE(status, GLenum(GL_FRAMEBUFFER_COMPLETE));
+        QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
 
         m_func->glBindFramebuffer(GL_FRAMEBUFFER, fbos[0]);
         m_func->glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, fboTextures[0], 0);
 
         status = m_func->glCheckFramebufferStatus(GL_FRAMEBUFFER);
-        QCOMPARE(status, GLenum(GL_FRAMEBUFFER_COMPLETE));
+        QVERIFY(status == GL_FRAMEBUFFER_COMPLETE);
 
         m_func->glEnable(GL_MULTISAMPLE);
         m_func->glClearColor(0.2f, 0.2f, 0.2f, 0.2f);
@@ -2111,8 +2031,8 @@ private Q_SLOTS:
 private:
     QScopedPointer<QWindow> m_window;
     QOpenGLContext m_glContext;
-    GraphicsHelperGL3_3 m_glHelper;
-    QOpenGLFunctions_3_3_Core *m_func = nullptr;
+    GraphicsHelperGL3_2 m_glHelper;
+    QOpenGLFunctions_3_2_Core *m_func = nullptr;
     bool m_initializationSuccessful = false;
 };
 
@@ -2128,7 +2048,7 @@ int main(int argc, char *argv[])
     QGuiApplication app(argc, argv);
     app.setAttribute(Qt::AA_Use96Dpi, true);
     QTEST_ADD_GPU_BLACKLIST_SUPPORT
-    tst_GraphicsHelperGL3_3 tc;
+    tst_GraphicsHelperGL3_2 tc;
     QTEST_SET_MAIN_SOURCE_PATH
     return QTest::qExec(&tc, argc, argv);
 #endif
@@ -2136,5 +2056,5 @@ int main(int argc, char *argv[])
 }
 
 #ifdef TEST_SHOULD_BE_PERFORMED
-#include "tst_graphicshelpergl3_3.moc"
+#include "tst_graphicshelpergl3_2.moc"
 #endif
